function [angularVelocityinBody, angularAccelerationinBody, angularJerkinBody] = calculateBodyAngularVelocity(RTS, frameRate)
%calculateBodyAngularVelocity - angular velocity in a fixed body frame from
%a sequence of rotation matrices representing the transformation from a lab
%inertial frame to the body frame.
%
% Inputs:
%    RTS - a rotation matrix time series: 3 x 3 x number of frames
%    frameNums - a vector of frame numbers
%    frameRate - the number of samples per second
%
% Outputs:
%    angularVelocityinBody - body angular velocity in
%    rad/s.
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

% From Diebel, J. (2006). Representing attitude: Euler angles, unit quaternions, and rotation vectors. Matrix 58, 15-16.

%RTS = allRFromWallToBody;

r0 = sqrt(1+RTS(1,1,:)+RTS(2,2,:)+RTS(3,3,:));
q0 = 1/2*[...
    squeeze(r0),...
    squeeze((RTS(2,3,:)-RTS(3,2,:))./r0),...
    squeeze((RTS(3,1,:)-RTS(1,3,:))./r0),...
    squeeze((RTS(1,2,:)-RTS(2,1,:))./r0)];

r1 = sqrt(1+RTS(1,1,:)-RTS(2,2,:)-RTS(3,3,:));
q1 = 1/2*[...
    squeeze((RTS(2,3,:)-RTS(3,2,:))./r1),...
    squeeze(r1),...
    squeeze((RTS(1,2,:)+RTS(2,1,:))./r1),...
    squeeze((RTS(3,1,:)+RTS(1,3,:))./r1)];

r2 = sqrt(1-RTS(1,1,:)+RTS(2,2,:)-RTS(3,3,:));
q2 = 1/2*[...
    squeeze((RTS(3,1,:)-RTS(1,3,:))./r2),...
    squeeze((RTS(1,2,:)+RTS(2,1,:))./r2),...
    squeeze(r2),...
    squeeze((RTS(2,3,:)+RTS(3,2,:))./r2)];

r3 = sqrt(1-RTS(1,1,:)-RTS(2,2,:)+RTS(3,3,:));
q3 = 1/2*[...
    squeeze((RTS(1,2,:)-RTS(2,1,:))./r3),...
    squeeze((RTS(3,1,:)+RTS(1,3,:))./r3),...
    squeeze((RTS(2,3,:)+RTS(3,2,:))./r3),...
    squeeze(r3)];

toPick = [...
    squeeze(RTS(2,2,:)>-RTS(3,3,:) & RTS(1,1,:)>-RTS(2,2,:) & RTS(1,1,:)>-RTS(3,3,:)),...
    squeeze(RTS(2,2,:)<-RTS(3,3,:) & RTS(1,1,:)>RTS(2,2,:) & RTS(1,1,:)>RTS(3,3,:)),...
    squeeze(RTS(2,2,:)>RTS(3,3,:) & RTS(1,1,:)<RTS(2,2,:) & RTS(1,1,:)<-RTS(3,3,:)),...
    squeeze(RTS(2,2,:)<RTS(3,3,:) & RTS(1,1,:)<-RTS(2,2,:) & RTS(1,1,:)<RTS(3,3,:)),...
    ];

% the quaternians
q = q0.*toPick(:,1) + q1.*toPick(:,2) + q2.*toPick(:,3) + q3.*toPick(:,4);
qdot = NaN(size(q));
qdotdot = NaN(size(q));
qdot3 = NaN(size(q));

% central difference derivatives
qdot(2:end-1,:) = (q(3:end,:)-q(1:end-2,:))*frameRate/2; % (f(t+1)-f(t-1))/(2h)
qdotdot(2:end-1,:) = (q(3:end,:)-2*q(2:end-1,:)+q(1:end-2,:))*frameRate^2; % (f(t+1)-2f(t)+f(t-1))/(h^2)
qdot3(3:end-2,:) = (q(5:end,:)-2*q(4:end-1,:)+2*q(2:end-3,:)-q(1:end-4,:))*frameRate^3/2; % (f(t+2)-2f(t+1)+2f(t-1)-f(t-2))/(2h^3)

angularVelocityinBody = NaN(size(q,1),3);
angularAccelerationinBody = NaN(size(q,1),3);
angularJerkinBody = NaN(size(q,1),3);
for i = 1:size(q,1)
    Wprime = [-q(i,2) q(i,1) q(i,4) -q(i,3);...
        -q(i,3) -q(i,4) q(i,1) q(i,2);...
        -q(i,4) q(i,3) -q(i,2) q(i,1)];
    
    angularVelocityinBody(i,:) = (2*Wprime*qdot(i,:)')';
    angularAccelerationinBody(i,:) = (2*Wprime*qdotdot(i,:)')';
    angularJerkinBody(i,:) = (2*Wprime*qdot3(i,:)')';
end

% Adjust components of angular val and acc so they point in intuitive
% directions
angularVelocityinBody(:,3) = -angularVelocityinBody(:,3);
angularAccelerationinBody(:,3) = -angularAccelerationinBody(:,3);
angularJerkinBody(:,3) = -angularJerkinBody(:,3);





