function trialTrackedDataStruct = calculateBodyKinematics(trialTrackedDataStruct, frameRate, createFigures)
%calculateBodyKinematics - calculates body position, velocity,
% and acceleration in the lab frame. Calls calculateBodyAngularVelocity.m
% to calculate body angular velocity and acceleration in body and lab
% frames.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing body kinematics data
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

%%
tds = trialTrackedDataStruct;

tdsFields = fields(tds);
trackedFeatureIdxs = cellfun(@(x) contains(x,'WingTip') || contains(x,'CM') || contains(x,'PVAinWall'), tdsFields);
featureNamesToRotate = tdsFields(trackedFeatureIdxs);
rotatedFeatureNamesBeez = cellfun(@(x)...
    strrep(strrep(x,'PVAinWall','PosinBeez'),'PosinWall','PosinBeez'), featureNamesToRotate,'UniformOutput',false);
for featureNum = 1:length(featureNamesToRotate)
    tds.(rotatedFeatureNamesBeez{featureNum}) =...
        tds.(featureNamesToRotate{featureNum});
end

rotatedFeatureNamesBody = cellfun(@(x)...
    strrep(strrep(x,'PVAinWall','PosinBody'),'PosinWall','PosinBody'), featureNamesToRotate,'UniformOutput',false);
for featureNum = 1:length(featureNamesToRotate)
    tds.(rotatedFeatureNamesBody{featureNum}) =...
        tds.(featureNamesToRotate{featureNum});
end

tds.tFromAbdToHeadinWall = - tds.smoothedAbdomenPVAinWall(:,1:4) + tds.smoothedHeadPVAinWall(:,1:4);
tds.tFromHeadToRWBinWall = - tds.smoothedHeadPVAinWall(:,1:4) + tds.smoothedRightWingBasePVAinWall(:,1:4);
tds.tFromHeadToLWBinWall = - tds.smoothedHeadPVAinWall(:,1:4) + tds.smoothedLeftWingBasePVAinWall(:,1:4);
    
% initialize the wing angle matrix [frameNums, yaw, pitch, roll]
bodyEulerAngles = [tds.frameNum,...
    NaN(size(tds.frameNum,1),3)];
allRFromWallToBeez = NaN(3,3,size(tds.frameNum,1));
allRFromBeezToBeeEL = NaN(3,3,size(tds.frameNum,1));
allRFromBeeELToBody = NaN(3,3,size(tds.frameNum,1));
allRFromWallToBody = NaN(3,3,size(tds.frameNum,1));

for frameNumber = 1:length(tds.frameNum)
    tFromWallToHeadinWall = tds.smoothedHeadPVAinWall(frameNumber,2:4);
    tFromAbdToHeadinWall = tds.tFromAbdToHeadinWall(frameNumber,2:4);
    tFromHeadToRWBinWall = tds.tFromHeadToRWBinWall(frameNumber,2:4);
    tFromHeadToLWBinWall = tds.tFromHeadToLWBinWall(frameNumber,2:4);
    
    if sum(isnan(tFromAbdToHeadinWall))>0
        bodyEulerAngles(frameNumber, 2:4) = NaN(1,3);
    else
        
        % 1. Rotate all body points to bodyZ frame
        yawRotVec = myvrrotvec([1 0 0], [tFromAbdToHeadinWall(1:2),0]);
        RFromWallToBeez = myvrrotvec2mat(yawRotVec);
        allRFromWallToBeez(:,:,frameNumber) = RFromWallToBeez;
        tFromAbdToHeadinBeez = tFromAbdToHeadinWall*RFromWallToBeez;
        tFromHeadToRWBinBeez = tFromHeadToRWBinWall*RFromWallToBeez;
        tFromHeadToLWBinBeez = tFromHeadToLWBinWall*RFromWallToBeez;
        
        % 2. Rotate all wing points to bodyEL frame
        elRotVec = myvrrotvec([1 0 0], tFromAbdToHeadinBeez);
        RFromBeezToBeeEL = myvrrotvec2mat(elRotVec);
        allRFromBeezToBeeEL(:,:,frameNumber) = RFromBeezToBeeEL;
        tFromAbdToHeadinBeeEL = tFromAbdToHeadinBeez*RFromBeezToBeeEL;
        tFromHeadToRWBinBeeEL = tFromHeadToRWBinBeez*RFromBeezToBeeEL;
        tFromHeadToLWBinBeeEL = tFromHeadToLWBinBeez*RFromBeezToBeeEL;
        
%         swelRotVec = myvrrotvec([1 0 0], tFromAbdToHeadinWall);
%         RFromWallToBeeEL = myvrrotvec2mat(swelRotVec);
%         tFromAbdToHeadinWallEL2 = tFromAbdToHeadinWall*RFromWallToBeeEL;
%         tFromHeadToRWBinWallEL2 = tFromHeadToRWBinWall*RFromWallToBeeEL;
%         tFromHeadToLWBinWallEL2 = tFromHeadToLWBinWall*RFromWallToBeeEL;
        
        % 3. Rotate all wing points to body frame
        % Want to rotate the vector from right wing base to left wing base
        % so that it points in the +y direction.
        if sum(isnan(tFromHeadToRWBinBeeEL))>0 || sum(isnan(tFromHeadToLWBinBeeEL))>0
            roRotVec = [NaN, NaN, NaN, NaN];
        else
            tFromRWBtoLWBinBeeEL = - tFromHeadToRWBinBeeEL + tFromHeadToLWBinBeeEL;
            tFromRWBtoLWBinBeeEL(1) = 0; % We want to rotate about the body axis
            roRotVec = myvrrotvec([0 1 0], tFromRWBtoLWBinBeeEL);
            RFromBeeELToBody = myvrrotvec2mat(roRotVec);
            allRFromBeeELToBody(:,:,frameNumber) = RFromBeeELToBody;
            allRFromWallToBody(:,:,frameNumber) = RFromWallToBeez*RFromBeezToBeeEL*RFromBeeELToBody;
            %tFromAbdToHeadinBody = tFromAbdToHeadinBeeEL*RFromBeeELToBody;
            %tFromHeadToRWBinBody = tFromHeadToRWBinBeeEL*RFromBeeELToBody;
            %tFromHeadToLWBinBody = tFromHeadToLWBinBeeEL*RFromBeeELToBody;
        end

        % We report body orientation angles as positive elevation for head
        % above the abdomen, positive yaw is to the left of straight ahead,
        % and positive roll is banking to the right
        bodyEulerAngles(frameNumber,2:4) = [...
            yawRotVec(4)*sign(yawRotVec(3)),...
            -elRotVec(4)*sign(elRotVec(2)),...
            roRotVec(4)*sign(roRotVec(1))]*180/pi;
        
        % Now calculate the position of all the points in Beez and Body
        % frames
        tFromWallToXYZinWall = cellfun(@(x) tds.(x)(frameNumber,2:4),featureNamesToRotate,'UniformOutput',false);
        %tFromWallToXYZinWall = cellfun(@(x) x(:,2:4), tFromWallToXYZinWall,'UniformOutput',false);
        tFromWallToXYZinWall = cell2mat(tFromWallToXYZinWall);
        
        % wing tip position is still in mm, but everything else is in
        % meters, so divide by 1000;
        wingTipIdx = cellfun(@(x) contains(x,'WingTip'), featureNamesToRotate);
        tFromWallToXYZinWall(wingTipIdx,:) = tFromWallToXYZinWall(wingTipIdx,:)/1000;
        cmIdx = cellfun(@(x) contains(x,'CM'), featureNamesToRotate);
        tFromWallToXYZinWall(cmIdx,:) = tFromWallToXYZinWall(cmIdx,:)/1000;
        
        % Derivation of tFromHeadToXYZinBeez:
%         tFromHeadToXYZinBeez = tFromHeadToWallinBeez + tFromWallToXYZinBeez
%             = tFromWallToXYZinWall*RFromWallToBeez - tFromWallToHeadinBeez
%             = (tFromWallToXYZinWall-tFromWallToHeadinWall)*RFromWallToBeez
        tFromHeadToXYZinBeez = (tFromWallToXYZinWall-tFromWallToHeadinWall)*RFromWallToBeez;
        tFromHeadToXYZinBody = tFromHeadToXYZinBeez*RFromBeezToBeeEL*RFromBeeELToBody;

        for featureNum = 1:length(featureNamesToRotate)
            tds.(rotatedFeatureNamesBeez{featureNum})(frameNumber,2:4) = tFromHeadToXYZinBeez(featureNum,:);
            tds.(rotatedFeatureNamesBody{featureNum})(frameNumber,2:4) = tFromHeadToXYZinBody(featureNum,:);
        end
        
        
    end
end

%k = infoStructEntry.zeroCrossingFrameNums
[bodyAngularVelocityinBody, bodyAngularAccelerationinBody] = calculateBodyAngularVelocity(allRFromWallToBody, frameRate);

if createFigures
    f1 = figure('units','normalized','outerposition',[0 0 1 1]);
    figure(f1)
    scale = 0.02;
    for k = 1:10:size(bodyEulerAngles,1)
        for i = 1:3
            subplot(2,4,i)
            plot(bodyEulerAngles(:,1),zeros(size(bodyEulerAngles,1),1),'black')
            hold on
            plot(bodyEulerAngles(:,1),-bodyEulerAngles(:,i+1))
            plot(bodyAngularVelocityinBody(:,i))
            plot(bodyEulerAngles(k,1),-bodyEulerAngles(k,i+1),'ro')
            hold off
        end
        subplot(2,4,5)
            plot(-[tds.smoothedHeadPosinBeez(k,3); tds.smoothedAbdomenPosinBeez(k,3)],...
                [tds.smoothedHeadPosinBeez(k,2); tds.smoothedAbdomenPosinBeez(k,2)],...
                'black');
            hold on
            % plot right wing
            plot(-[tds.smoothedRightWingBasePosinBeez(k,3); tds.rawRightWingTipPosinBeez(k,3)],...
                [tds.smoothedRightWingBasePosinBeez(k,2); tds.rawRightWingTipPosinBeez(k,2)],...
                'r');
            % plot left wing
            plot(-[tds.smoothedLeftWingBasePosinBeez(k,3); tds.rawLeftWingTipPosinBeez(k,3)],...
                [tds.smoothedLeftWingBasePosinBeez(k,2); tds.rawLeftWingTipPosinBeez(k,2)],...
                'b');
            hold off
            axis([-1,1, -1,1].*scale)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        subplot(2,4,6)
            plot(-[tds.smoothedHeadPosinBeez(k,3); tds.smoothedAbdomenPosinBeez(k,3)],...
                [tds.smoothedHeadPosinBeez(k,4); tds.smoothedAbdomenPosinBeez(k,4)],...
                'black');
            hold on
            % plot right wing
            plot(-[tds.smoothedRightWingBasePosinBeez(k,3); tds.rawRightWingTipPosinBeez(k,3)],...
                [tds.smoothedRightWingBasePosinBeez(k,4); tds.rawRightWingTipPosinBeez(k,4)],...
                'r');
            % plot left wing
            plot(-[tds.smoothedLeftWingBasePosinBeez(k,3); tds.rawLeftWingTipPosinBeez(k,3)],...
                [tds.smoothedLeftWingBasePosinBeez(k,4); tds.rawLeftWingTipPosinBeez(k,4)],...
                'b');
            hold off
            axis([-1,1, -1,1].*scale)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        subplot(2,4,7)
            plot([tds.smoothedHeadPosinBeez(k,2); tds.smoothedAbdomenPosinBeez(k,2)],...
                [tds.smoothedHeadPosinBeez(k,4); tds.smoothedAbdomenPosinBeez(k,4)],...
                'black');
            hold on
            % plot right wing
            plot([tds.smoothedRightWingBasePosinBeez(k,2); tds.rawRightWingTipPosinBeez(k,2)],...
                [tds.smoothedRightWingBasePosinBeez(k,4); tds.rawRightWingTipPosinBeez(k,4)],...
                'r');
            % plot left wing
            plot([tds.smoothedLeftWingBasePosinBeez(k,2); tds.rawLeftWingTipPosinBeez(k,2)],...
                [tds.smoothedLeftWingBasePosinBeez(k,4); tds.rawLeftWingTipPosinBeez(k,4)],...
                'b');
            hold off
            axis([-1,1, -1,1].*scale)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
            
        subplot(2,4,4)
            
            % plot body axis
            plot3([tds.smoothedHeadPosinBeez(k,2); tds.smoothedAbdomenPosinBeez(k,2)],...
                [tds.smoothedHeadPosinBeez(k,3); tds.smoothedAbdomenPosinBeez(k,3)],...
                [tds.smoothedHeadPosinBeez(k,4); tds.smoothedAbdomenPosinBeez(k,4)],...
                'black');
            hold on
            % plot right wing
            plot3([tds.smoothedRightWingBasePosinBeez(k,2); tds.rawRightWingTipPosinBeez(k,2)],...
                [tds.smoothedRightWingBasePosinBeez(k,3); tds.rawRightWingTipPosinBeez(k,3)],...
                [tds.smoothedRightWingBasePosinBeez(k,4); tds.rawRightWingTipPosinBeez(k,4)],...
                'r');
            % plot left wing
            plot3([tds.smoothedLeftWingBasePosinBeez(k,2); tds.rawLeftWingTipPosinBeez(k,2)],...
                [tds.smoothedLeftWingBasePosinBeez(k,3); tds.rawLeftWingTipPosinBeez(k,3)],...
                [tds.smoothedLeftWingBasePosinBeez(k,4); tds.rawLeftWingTipPosinBeez(k,4)],...
                'b');

            hold off
            xlabel('x')
            ylabel('y')
            zlabel('z')
            axis([-1,1, -1,1, -1,1].*scale)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
            
        subplot(2,4,8)
            
            % plot body axis
            plot3([tds.smoothedHeadPVAinWall(k,2); tds.smoothedAbdomenPVAinWall(k,2)],...
                [tds.smoothedHeadPVAinWall(k,3); tds.smoothedAbdomenPVAinWall(k,3)],...
                [tds.smoothedHeadPVAinWall(k,4); tds.smoothedAbdomenPVAinWall(k,4)],...
                'black');
            xlabel('x')
            ylabel('y')
            zlabel('z')
            %axis([-1,1, -1,1, -1,1].*scale/1000)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        %clear 'view';
        %view(ceil(180/265*k),30);
        pause(0.05);
        drawnow limitrate
    end
end

%%
tds.bodyEulerAngles = bodyEulerAngles;
tds.allRFromWallToBeez = allRFromWallToBeez;
tds.allRFromBeezToBeeEL = allRFromBeezToBeeEL;
tds.allRFromBeeELToBody = allRFromBeeELToBody;
tds.allRFromWallToBody = allRFromWallToBody;
tds.bodyAngularVelocityinBody = bodyAngularVelocityinBody;
tds.bodyAngularAccelerationinBody = bodyAngularAccelerationinBody;
trialTrackedDataStruct = tds;

