function infoStructEntry = calculateKinematicsMeansForTimeSegments(infoStructEntry, frameRate)
%calculateKinematicsMeansForTimingSegments - calculates body and wing kinematics
% across several different time periods defined by the bill crossing the
% gust midline and by the tail crossing the gust midline or by the tail
% leaving the gust.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    frameRate - camera frame rate in frames per second
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing wing kinematics data
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

%infoStructEntry = trialInfoStruct(22);

ise = infoStructEntry;
tds = infoStructEntry.trialData;

% Calculate time segments
% beforeEntry: t < tBBCrossMidline
% options for calculating means of kinematics
    % bodyInGust: t > tBBCrossMidline & t < tTCCrossMidline
    % bodyOrTailInGust: t > tBBCrossMidline & t < tTCLeaveGust
    % justTailInGust: t > tTCCrossMidline & t < tTCLeaveGust
% options for calculating post means
    % postBody: t > tTCCrossMidline
    % postBodyAndTail: t > tTCLeaveGust

% Make a variable that keeps track of real time relative to tBBCrossMidline
tds.tRelBBCross = (tds.frameNum-ise.tBBCrossMidline)/frameRate;

% Set up a data struct to keep track of which means correspond to which
% time segments.
% Struct outline: tds.segmentMeans.featureName.segmentName will contain a
% 3 element vector [segment start time relative to BBCrossMidline,
%                   segment end time relative to BBCrossMidline,
%                   mean value for the feature over the corresponding segment]
featureNames = {'birdElevation', 'instantaneousVX', 'tailElevationInBirdZ', 'tailElevationInBody', ...
    'tailExtensionAngle', 'bodyAngularVelocityinBody', 'bodyAngularVelocityinLab', ...
    'bodyAngularAccelerationinBody', 'bodyAngularAccelerationinLab', 'rawWingSweepAngleInBirdZ', ...
    'rawWingElevationAngleInBirdZ', 'rawWingSweepAngleInBody', 'rawWingElevationAngleInBody', ...
    'proportionSweptBackAndElevatedInBirdZ', 'proportionSweptBackAndElevatedInBody', 'proportionElevatedInBody', ...
    'strokeSweepAmplitudeInBirdZ', 'wingbeatFrequency', 'strokeSweepOffsetInBirdZ'};
% Be = before, Af = after, En = entry, Cm = cross midline, Lg = leaves gust
% Bb = bill base, Tc = tail center
segmentNameInfo = 'Be = before, Af = after, Cm = cross midline, Lg = leaves gust, Bb = bill base, Tc = tail center';
segmentContentsInfo = '[segment start time realative to BBCrossMidline, segment end time relative to BBCrossMidline, mean value for the feature over corresponding segment]';
segmentNames = {'BeBbCm', 'AfBbCmBeTcCm', 'AfBbCmBeTcLg', 'AfTcCmBeTcLg', 'AfTcCm', 'AfTcLg'};
segmentRanges = {[-inf, ise.tBBCrossMidline], [ise.tBBCrossMidline, ise.tTCCrossMidlineFrameNum], ...
    [ise.tBBCrossMidline, ise.tTCLeaveGustFrameNum], [ise.tTCCrossMidlineFrameNum, ise.tTCLeaveGustFrameNum], ...
    [ise.tTCCrossMidlineFrameNum, inf], [ise.tTCLeaveGustFrameNum, inf]};

% Account for three control trials which weren't tracked past the tail crossing the midline
% Note for these three trials, the means for tBBCrossMidline to
% lastRealFrame will be used for all time segments except BeBbCm.
if isnan(ise.tTCCrossMidlineFrameNum)
    for segNum = 2:length(segmentRanges)
        segmentRanges{segNum} = [ise.tBBCrossMidline, ise.lastRealFrame];
    end
end

segmentMeans = struct();
segmentMeans.segmentNameInfo = segmentNameInfo;
segmentMeans.segmentContentsInfo = segmentContentsInfo;
for featNum = 1:length(featureNames)
    for segNum = 1:length(segmentNames)
        % calculate timing relative to bill base cross midline
        segTimes = (segmentRanges{segNum}-ise.tBBCrossMidline)/frameRate;
        segmentMeans.(featureNames{featNum}).(segmentNames{segNum}) = [segTimes, NaN];
    end
end

for segNum = 1:length(segmentNames)
    % write a vector of the frames we want to average over
    tempTimes = (tds.birdElevation(:,1) >= segmentRanges{segNum}(1) & tds.birdElevation(:,1) < segmentRanges{segNum}(2)) & ~isnan(tds.birdElevation(:,2));
    % use the vector to pick out the frames of each kinematic quantity (one in each
    % column).
    tempData = [tds.birdElevation(:,2) ...
        tds.tFromANCToBBsmoothedPVAInLab(:,5) ...
        tds.tailElevationInBirdZ(:,2) ...
        tds.tailElevationInBody(:,2) tds.tailExtensionAngle(:,2) ...
        tds.bodyAngularVelocityinBody(:,3)*180/pi tds.bodyAngularVelocityinLab(:,3)*180/pi ...
        tds.bodyAngularAccelerationinBody(:,3)*180/pi tds.bodyAngularAccelerationinLab(:,3)*180/pi ...
        tds.rawWingSweepAngle(:,2) tds.rawWingElevationAngle(:,2) ...
        tds.rawWingSweepAngleInBody(:,2) tds.rawWingElevationAngleInBody(:,2) ...
        tds.rawWingSweepAngle(:,2)<=0 & tds.rawWingElevationAngle(:,2)>=0 & ~isnan(tds.rawWingElevationAngle(:,2)) ...
        tds.rawWingSweepAngleInBody(:,2)<=0 & tds.rawWingElevationAngleInBody(:,2)>=0 & ~isnan(tds.rawWingElevationAngleInBody(:,2)) ...
        tds.rawWingElevationAngleInBody(:,2)>=30 & ~isnan(tds.rawWingElevationAngleInBody(:,2)) ...
        ].*tempTimes;
    % calculate the mean
    tMeans = nansum(tempData,1)./sum(tempTimes);
    
    for featNum = 1:length(tMeans)
        segmentMeans.(featureNames{featNum}).(segmentNames{segNum})(3) = tMeans(featNum);
    end
    
    % wing stroke parameters are a little different
    % strokeDataOverSegment contains:
    % 'time','amplitude','frequency','phase','offset','highs','lows','meanFitError'
    tempSData = tds.strokeDataOverSegment(:,[1 2 3 5]);
    tempSTimes = (tempSData(:,1) > segmentRanges{segNum}(1)/frameRate) & (tempSData(:,1) < segmentRanges{segNum}(2)/frameRate);
    tSMeans = nanmean(tempSData(tempSTimes, [2 3 4]),1);
    
    for featNum = 1:length(tSMeans)
        segmentMeans.(featureNames{featNum+length(tMeans)}).(segmentNames{segNum})(3) = tSMeans(featNum);
    end
    
end

ise.segmentMeans = segmentMeans;

% Calculating kinematics at specific time points:
% Can define "recovery phase" using
% minBodyAngle
%   tMinBodyAngle(relative to tBBCrossMidline), minBodyAngleAtEnd
% minAngularVelocity (i.e. maximum downward pitch rotation rate)
%   tMinPitchDownAngularVelocity (relative to tBBCrossMidline)
% minAngularAcceleration (i.e. maximum downward pitch accel)
%   tMaxPitchDownAngularAcceleration (relative to tBBCrossMidline)

ise.vxBBAtBBCrossMidline = fnval(tds.tFromANCToBBsplineInLab.vx,ise.tBBCrossMidline)*500/100;
ise.vzBBAtBBCrossMidline = fnval(tds.tFromANCToBBsplineInLab.vz,ise.tBBCrossMidline)*500/100;
ise.bodyAngleAtBBEnterGust = tds.birdElevation(round(ise.tBBEnterGustFrameNum),2);
ise.bodyAngleAtBBCrossMidline = tds.birdElevation(round(ise.tBBCrossMidline),2);
ise.bodyAngularAccelerationInBodyAtBBEnterGust = tds.bodyAngularAccelerationinBody(round(ise.tBBEnterGustFrameNum),3)*180/pi;
ise.bodyAngularAccelerationInBodyAtBBCrossMidline = tds.bodyAngularAccelerationinBody(round(ise.tBBCrossMidline),3)*180/pi;

% calculate bill base and body kinematics at time points of interest
timePointNames = {'TCCrossMidline','TCLeaveGust'};
for tpNum = 1:length(timePointNames)
    timePoint = ise.(['t' timePointNames{tpNum} 'FrameNum']);
    if isnan(timePoint)
        timePoint = ise.lastRealFrame;
    end
    ise.(['zBBAT' timePointNames{tpNum}]) = fnval(tds.tFromANCToBBsplineInLab.pz,timePoint);
    ise.(['vxBBAT' timePointNames{tpNum}]) = fnval(tds.tFromANCToBBsplineInLab.vx,timePoint)*500/100;
    ise.(['vzBBAT' timePointNames{tpNum}]) = fnval(tds.tFromANCToBBsplineInLab.vz,timePoint)*500/100;
    ise.(['bodyAngleAT' timePointNames{tpNum}]) = tds.birdElevation(round(timePoint),2);
end

% minimum body angle
[minBodyAngle, mbaidx] = min(tds.birdElevation(:,2));
ise.minBodyAngle = minBodyAngle;
ise.tminBodyAngle.value = tds.tRelBBCross(mbaidx);
for segNum = [1 2 4 6] % we only care about the most detailed time segments (3 and 5 overlap with 2, 4, and 6)
    if mbaidx >= segmentRanges{segNum}(1) && mbaidx < segmentRanges{segNum}(2)
        if mbaidx == ise.lastRealFrame
            ise.tminBodyAngle.inSegment = 'AfTrEnd';
        else
            ise.tminBodyAngle.inSegment = segmentNames{segNum};
        end
    elseif segNum == 6 && mbaidx >= segmentRanges{segNum}(2)
        ise.tminBodyAngle.inSegment = 'AfTrEnd';
    end
end

% minimum angular velocity about pitch axis (i.e. most downward pitch rate)
[minBodyAngVel, mbavidx] = min(tds.bodyAngularVelocityinBody(:,3));
ise.minBodyAngularVelocity = minBodyAngVel*180/pi; % convert from rad/s to deg/s
ise.tminBodyAngularVelocity.value = tds.tRelBBCross(mbavidx);
for segNum = [1 2 4 6]
    if mbavidx >= segmentRanges{segNum}(1) && mbavidx < segmentRanges{segNum}(2)
        if mbavidx == ise.lastRealFrame
            ise.tminBodyAngularVelocity.inSegment = 'AfTrEnd';
        else
            ise.tminBodyAngularVelocity.inSegment = segmentNames{segNum};
        end
    elseif segNum == 6 && mbavidx >= segmentRanges{segNum}(2)
        ise.tminBodyAngularVelocity.inSegment = 'AfTrEnd';
    end
end

% minimum angular acceleration about pitch axis (i.e. most downward pitch acceleration)
[minBodyAngAcc, mbaaidx] = min(tds.bodyAngularAccelerationinBody(:,3));
ise.minBodyAngularAcceleration = minBodyAngAcc*180/pi; % convert from rad/s to deg/s
ise.tminBodyAngularAcceleration.value = tds.tRelBBCross(mbaaidx);
for segNum = [1 2 4 6]
    if mbaaidx >= segmentRanges{segNum}(1) && mbaaidx < segmentRanges{segNum}(2)
        if mbaaidx == ise.lastRealFrame
            ise.tminBodyAngularAcceleration.inSegment = 'AfTrEnd';
        else
            ise.tminBodyAngularAcceleration.inSegment = segmentNames{segNum};
        end
    elseif segNum == 6 && mbaaidx >= segmentRanges{segNum}(2)
        ise.tminBodyAngularAcceleration.inSegment = 'AfTrEnd';
    end
end

ise.tBBEnterGust = (ise.tBBEnterGustFrameNum - ise.tBBCrossMidline)/frameRate;
ise.tTCCrossMidline = (ise.tTCCrossMidlineFrameNum - ise.tBBCrossMidline)/frameRate;
ise.tTCLeaveGust = (ise.tTCLeaveGustFrameNum - ise.tBBCrossMidline)/frameRate;

% Copy results to original
ise.trialData = tds;
infoStructEntry = ise;

