function infoStructEntry = calculateTransitPhaseTimings(infoStructEntry)
%calculateTransitPhaseTimings - calculates the phases of a transit
%defined by:
% 0: before bill base crosses gust midline
% 1: after bill base crosses midline but before tail center crosses midline
% 2: after tail center crosses midline but before tail leaves gust
% 3: after tail center leaves gust
%
% Inputs:
%    filename - filename of data file
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing transit timings
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

% infoStructEntry = trialInfoStruct(2);

tds = infoStructEntry.trialData;

% Calculate bill base crossing midline timing and position
tBBCrossMidline = fnzeros(tds.tFromANCToBBsplineInLab.px);
infoStructEntry.tBBCrossMidline = tBBCrossMidline(1,:);
infoStructEntry.zBBAtBBCrossMidline = fnval(tds.tFromANCToBBsplineInLab.pz,infoStructEntry.tBBCrossMidline);
infoStructEntry.zBBAtBBCrossMidlineFrame = fnval(tds.tFromANCToBBsplineInLab.pz,infoStructEntry.zeroCrossingFrameNums);
infoStructEntry.yBBAtBBCrossMidline = fnval(tds.tFromANCToBBsplineInLab.py,infoStructEntry.tBBCrossMidline);

% Calculate bill base entering gust timing and position
tBBEnterGustFrameNum = atan(abs(tds.tFromANCToBBsmoothedPVAInLab(:,2))./tds.tFromANCToBBsmoothedPVAInLab(:,4))*180/pi;
tBBEnterGustFrameNum = find(tBBEnterGustFrameNum<10);
if ~isempty(tBBEnterGustFrameNum)
    infoStructEntry.tBBEnterGustFrameNum = tBBEnterGustFrameNum(1);
else
    infoStructEntry.tBBEnterGustFrameNum = NaN;
end

% Calculate tail center crossing midline timing and position
tTCCrossMidlineFrameNum = fnzeros(tds.tFromANCToTCsplineInLab.px);
tTCLeaveGustFrameNum = atan(abs(tds.tFromANCToTCsmoothedPVAInLab(:,2))./tds.tFromANCToTCsmoothedPVAInLab(:,4))*180/pi;
tTCLeaveGustFrameNum = find(tTCLeaveGustFrameNum<10);
if ~isempty(tTCLeaveGustFrameNum)
    tTCLeaveGustFrameNum = tTCLeaveGustFrameNum(end);
else
    infoStructEntry.tTCLeaveGustFrameNum = NaN;
end
if isempty(tTCCrossMidlineFrameNum(1,:))
    infoStructEntry.tTCCrossMidlineFrameNum = NaN;
    infoStructEntry.tTCLeaveGustFrameNum = NaN;
else
    infoStructEntry.tTCCrossMidlineFrameNum = tTCCrossMidlineFrameNum(1,:);
    if tTCLeaveGustFrameNum == infoStructEntry.lastRealFrame
        infoStructEntry.tTCLeaveGustFrameNum = NaN;
    else
        infoStructEntry.tTCLeaveGustFrameNum = tTCLeaveGustFrameNum;
    end
end

% Calculate tail center leaving gust plane
% Angle from gust center to tail center > 90 deg.

% Transit phases are:
% 0: before bill base crosses gust midline
% 1: after bill base crosses midline but before tail center crosses midline
% 2: after tail center crosses midline
% 3: after tail center leaves gust
transitPhase = tds.frameNum;
transitPhase(tds.frameNum <= tds.zeroCrossingFrameNums) = 0;
transitPhase(tds.frameNum > tds.zeroCrossingFrameNums & tds.frameNum <= nanmin([infoStructEntry.tTCCrossMidlineFrameNum, infoStructEntry.tTCLeaveGustFrameNum, infoStructEntry.lastRealFrame,])) = 1;
if ~isnan(infoStructEntry.tTCCrossMidlineFrameNum)
    transitPhase(tds.frameNum > infoStructEntry.tTCCrossMidlineFrameNum & tds.frameNum <= nanmin([infoStructEntry.tTCLeaveGustFrameNum, infoStructEntry.lastRealFrame,])) = 2;
end
if ~isnan(infoStructEntry.tTCLeaveGustFrameNum)
    transitPhase(tds.frameNum > infoStructEntry.tTCLeaveGustFrameNum) = 3;
end
transitPhase(tds.frameNum<infoStructEntry.firstRealFrame | tds.frameNum>infoStructEntry.lastRealFrame) = NaN;
tds.transitPhase = transitPhase;

infoStructEntry.trialData = tds;

end