function infoStructEntry = calculateWingKinematics(infoStructEntry, numHCLsPerFit, frameRate, createFigures)
%calculateWingKinematics - calculates wing kinematics with respect to the
% horizontal plane (BirdZ frame).
% Stroke sweep/positional angle, stroke deviation/elevation angle, and wing
% rotation angle (about its axis) are calculated for each frame.  For each
% stroke we caluclate stroke amplitude and offset, stroke deviation 
% amplitude and stroke frequency.  Many trials do not have defined stroke
% patterns, so we also calculate wing holding, defined as the percentage of
% time the wings were held both backward and elevated.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing wing kinematics data
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

%%
% [themin, minidx] = min(arrayfun(@(x) x.lastRealFrame-x.firstRealFrame, trialInfoStruct))
% [themax, maxidx] = max(arrayfun(@(x) x.lastRealFrame-x.firstRealFrame, trialInfoStruct))

% find(arrayfun(@(x) min(x.trialData.strokeSummaryStruct.amplitude.data(:,2)) < 0,trialInfoStruct))
% infoStructEntry = trialInfoStruct(76);

tds = infoStructEntry.trialData;

% Fake data for testing sweep
% ts = tds.tFromWBToWTrawPInBirdZ(:,1);
% sweepAngSim = 3*pi/4*sin(ts/500*20*2*pi)-pi/2;
% plot(sweepAngSim)
% tds.tFromWBToWTrawPInBirdZ(:,2:4) = [4*cos(sweepAngSim), 4*sin(sweepAngSim), zeros(length(sweepAngSim),1)];
% plot(-tds.tFromWBToWTrawPInBirdZ(:,3),tds.tFromWBToWTrawPInBirdZ(:,2))
% 
% ts = tds.tFromWBToWTrawPInBirdZ(:,1);
% elevAngSim = 3*pi/4*cos(ts/500*10*2*pi)+pi;
% plot(elevAngSim)
% tds.tFromWBToWTrawPInBirdZ(:,2:4) = [4*cos(sweepAngSim), 4*sin(sweepAngSim), 1*cos(elevAngSim)];
% plot3(tds.tFromWBToWTrawPInBirdZ(:,2),tds.tFromWBToWTrawPInBirdZ(:,3),tds.tFromWBToWTrawPInBirdZ(:,4))
% pbaspect([1 1 1]);
% daspect([1 1 1]);
% 
% ts = tds.tFromWBToWTrawPInBirdZ(:,1);
% elevAngSim = 1*pi/4*cos(ts/500*20*2*pi)+pi;
% plot(elevAngSim)
% tds.tFromWBToWTrawPInBirdZ(:,2:4) = [1*cos(sweepAngSim), 1*sin(sweepAngSim), 1*sin(elevAngSim)];
% plot3(tds.tFromWBToWTrawPInBirdZ(:,2),tds.tFromWBToWTrawPInBirdZ(:,3),tds.tFromWBToWTrawPInBirdZ(:,4))
% pbaspect([1 1 1]);
% daspect([1 1 1]);

% If right wing was tracked, reflect about the body axis so angles will
% work out. Note, only one wing (closest to side camera) per transit was tracked.
% Don't need to do this if x coordinate was reflected in
% loadAndProcessTrial.m.
switchRL = false; % strcmp(infoStructEntry.flightDirection,'dB');

tds.tFromWBToFF4rawPInBirdZ = tds.tFromBBToFF4rawPInBirdZ-tds.tFromBBToWBsmoothedPInBirdZ;
    
% initialize the wing angle matrix [frameNums, sweep, deviation, rotation]
wingAngles = [tds.frameNum,...
    NaN(size(tds.frameNum,1),3)];

% tempStuff = [tds.frameNum,...
%     NaN(size(tds.frameNum,1),5)];

for frameNumber = 1:length(tds.frameNum)
    tFromWBToWTinBirdZ = tds.tFromWBToWTrawPInBirdZ(frameNumber,2:4);
    tFromWBToFF4inBirdZ = tds.tFromWBToFF4rawPInBirdZ(frameNumber,2:4);
    
    if switchRL
        tFromWBToWTinBirdZ(2) = -tFromWBToWTinBirdZ(2);
        tFromWBToFF4inBirdZ(2) = -tFromWBToFF4inBirdZ(2);
    end
    
    %tempStuff(frameNumber,2:4) = tFromWBToWTinBirdZ;
    
    if sum(isnan(tFromWBToWTinBirdZ))>0
        wingAngles(frameNumber, 2:4) = NaN(1,3);
    else
        
        % 1. Rotate all wing points to wingSW frame
        swRotVec = vrrotvec([0 1 0], [tFromWBToWTinBirdZ(1:2),0]);
        RFromBirdZToWingSW = vrrotvec2mat(swRotVec);
        tFromWBToWTinWingSW = tFromWBToWTinBirdZ*RFromBirdZToWingSW;
        tFromWBToFF4inWingSW = tFromWBToFF4inBirdZ*RFromBirdZToWingSW;
        
        % 2. Rotate all wing points to wingEL frame
        elRotVec = vrrotvec([0 1 0], tFromWBToWTinWingSW);
        RFromWingSWToWingEL = vrrotvec2mat(elRotVec);
        tFromWBToWTinWingEL = tFromWBToWTinWingSW*RFromWingSWToWingEL;
        tFromWBToFF4inWingEL = tFromWBToFF4inWingSW*RFromWingSWToWingEL;
        
        swelRotVec = vrrotvec([0 1 0], tFromWBToWTinBirdZ);
        RFromBirdZToWingEL = vrrotvec2mat(swelRotVec);
        tFromWBToWTinWingEL2 = tFromWBToWTinBirdZ*RFromBirdZToWingEL;
        tFromWBToFF4inWingEL2 = tFromWBToFF4inBirdZ*RFromBirdZToWingEL;
        
        % 3. Rotate all wing points to wing frame
        % Want to rotate the normal of the wing plane so that it points in
        % the +z direction.
        if sum(isnan(tFromWBToFF4inBirdZ))>0
            roRotVec = [NaN, NaN, NaN, NaN];
        else
            plNorm = cross(tFromWBToWTinWingEL, tFromWBToFF4inWingEL);
            roRotVec = vrrotvec([0 0 1], plNorm);
            RFromWingELToWing = vrrotvec2mat(roRotVec);
            tFromWBToWTinWing = tFromWBToWTinWingEL*RFromWingELToWing;
            tFromWBToFF4inWing = tFromWBToFF4inWingEL*RFromWingELToWing;
        end
        
        wingAngles(frameNumber,2:4) = [...
            -swRotVec(4)*sign(swRotVec(3)),...
            elRotVec(4)*sign(elRotVec(1)),...
            -roRotVec(4)*sign(roRotVec(2))]*180/pi;
        
%         tempStuff(frameNumber,5) = -swRotVec(4)*sign(swRotVec(3));
%         tempStuff(frameNumber,6) = elRotVec(4)*sign(elRotVec(1));
%         
%         k = frameNumber;
%         subplot(1,3,1)
%         plot(tempStuff(:,1), tempStuff(:,5))
%         hold on
%         plot(tempStuff(k,1), tempStuff(k,5),'ro')
%         hold off
%         subplot(1,3,2)
%         plot(tempStuff(:,1), tempStuff(:,6))
%         hold on
%         plot(tempStuff(k,1), tempStuff(k,6),'ro')
%         hold off
%         subplot(1,3,3)
%         plot3(tempStuff(:,2),tempStuff(:,3),tempStuff(:,4))
%         hold on
%         plot3(tempStuff(k,2),tempStuff(k,3),tempStuff(k,4),'ro')
%         hold off
%         pbaspect([1 1 1]);
%         daspect([1 1 1]);
%         pause(0.125);
%         drawnow limitrate
        
        % "Angle of attack" (ignoring that the wing is upside down)
        % if wingAngles(frameNumber,4) > 90
        %    wingAngles(frameNumber,4) = 180-wingAngles(frameNumber,4);
        % end
         
        % can also get the wing sweep and elevation like this:
        % [wingH,wingE,~] = cart2sph(tFromWBToWTinBirdZ(1),tFromWBToWTinBirdZ(2),tFromWBToWTinBirdZ(3));
        % wingAngles(frameNumber,2:3) = [-wingH+pi/2, wingE]*180/pi;
        
    end
end

%k = infoStructEntry.zeroCrossingFrameNums

if createFigures
    figure();
    for k = infoStructEntry.firstRealFrame:infoStructEntry.lastRealFrame
        for i = 1:3
            subplot(2,4,i)
            plot(wingAngles(:,1),wingAngles(:,i+1))
            hold on
            plot(wingAngles(k,1),wingAngles(k,i+1),'ro')
            hold off
        end
        subplot(2,4,5)
            plot(-[tds.tFromBBToBBsmoothedPInBirdZ(k,3); tds.tFromBBToRUsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToBBsmoothedPInBirdZ(k,2); tds.tFromBBToRUsmoothedPInBirdZ(k,2)],...
                'black');
            hold on
            % plot right wing
            plot(-[tds.tFromBBToWBsmoothedPInBirdZ(k,3); tds.tFromBBToWTrawPInBirdZ(k,3);tds.tFromBBToFF4rawPInBirdZ(k,3);tds.tFromBBToWBsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToWBsmoothedPInBirdZ(k,2); tds.tFromBBToWTrawPInBirdZ(k,2);tds.tFromBBToFF4rawPInBirdZ(k,2);tds.tFromBBToWBsmoothedPInBirdZ(k,2)],...
                'r');
            hold off
            axis([-1,1, -1,1].*7)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        subplot(2,4,6)
            plot(-[tds.tFromBBToBBsmoothedPInBirdZ(k,3); tds.tFromBBToRUsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToBBsmoothedPInBirdZ(k,4); tds.tFromBBToRUsmoothedPInBirdZ(k,4)],...
                'black');
            hold on
            % plot right wing
            plot(-[tds.tFromBBToWBsmoothedPInBirdZ(k,3); tds.tFromBBToWTrawPInBirdZ(k,3);tds.tFromBBToFF4rawPInBirdZ(k,3);tds.tFromBBToWBsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToWBsmoothedPInBirdZ(k,4); tds.tFromBBToWTrawPInBirdZ(k,4);tds.tFromBBToFF4rawPInBirdZ(k,4);tds.tFromBBToWBsmoothedPInBirdZ(k,4)],...
                'r');
            hold off
            axis([-1,1, -1,1].*7)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        subplot(2,4,7)
            plot([tds.tFromBBToBBsmoothedPInBirdZ(k,2); tds.tFromBBToRUsmoothedPInBirdZ(k,2)],...
                [tds.tFromBBToBBsmoothedPInBirdZ(k,4); tds.tFromBBToRUsmoothedPInBirdZ(k,4)],...
                'black');
            hold on
            % plot right wing
            plot([tds.tFromBBToWBsmoothedPInBirdZ(k,2); tds.tFromBBToWTrawPInBirdZ(k,2);tds.tFromBBToFF4rawPInBirdZ(k,2);tds.tFromBBToWBsmoothedPInBirdZ(k,2)],...
                [tds.tFromBBToWBsmoothedPInBirdZ(k,4); tds.tFromBBToWTrawPInBirdZ(k,4);tds.tFromBBToFF4rawPInBirdZ(k,4);tds.tFromBBToWBsmoothedPInBirdZ(k,4)],...
                'r');
            hold off
            axis([-1,1, -1,1].*7)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
            
        subplot(2,4,[4 8])
            % plot air knife
            % air knife is 16 cm long oriented along Lab_y
            tFromANCToPTSinLab = [0 -8 0; 0 8 0; 0 8 24; 0 -8 24; 0 -8 0];
            %tFromBBToPTSinBirdZ = tFromBBToANCinBirdZ + tFromANCToPTSinBirdZ;
            tFromBBToPTSinBirdZ = (-tds.tFromANCToBBsmoothedPVAInLab(k,2:4) + tFromANCToPTSinLab)*tds.RFromLabToBirdZ(:,:,k);
            fill3(tFromBBToPTSinBirdZ(:,1),tFromBBToPTSinBirdZ(:,2),tFromBBToPTSinBirdZ(:,3),'blue',...
                'FaceColor','b','FaceAlpha',.2,'EdgeAlpha',1.0);
            hold on
            
            % plot body axis
            plot3([tds.tFromBBToBBsmoothedPInBirdZ(k,2); tds.tFromBBToRUsmoothedPInBirdZ(k,2)],...
                [tds.tFromBBToBBsmoothedPInBirdZ(k,3); tds.tFromBBToRUsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToBBsmoothedPInBirdZ(k,4); tds.tFromBBToRUsmoothedPInBirdZ(k,4)],...
                'black');
            hold on
            % plot right wing
            plot3([tds.tFromBBToWBsmoothedPInBirdZ(k,2); tds.tFromBBToWTrawPInBirdZ(k,2);tds.tFromBBToFF4rawPInBirdZ(k,2);tds.tFromBBToWBsmoothedPInBirdZ(k,2)],...
                [tds.tFromBBToWBsmoothedPInBirdZ(k,3); tds.tFromBBToWTrawPInBirdZ(k,3);tds.tFromBBToFF4rawPInBirdZ(k,3);tds.tFromBBToWBsmoothedPInBirdZ(k,3)],...
                [tds.tFromBBToWBsmoothedPInBirdZ(k,4); tds.tFromBBToWTrawPInBirdZ(k,4);tds.tFromBBToFF4rawPInBirdZ(k,4);tds.tFromBBToWBsmoothedPInBirdZ(k,4)],...
                'r');

            hold off
            xlabel('x')
            ylabel('y')
            zlabel('z')
            axis([-1,1, -1,1, -1,1].*7)
            pbaspect([1 1 1]);
            daspect([1 1 1]);
        %clear 'view';
        %view(ceil(180/265*k),30);
        pause(0.125);
        drawnow limitrate
    end
end

%% Calculate wing stroke amplitude, frequency and offset for sweep angle
% Calculate trajectories and also means for various timing segments
% [frameNums, sweep, deviation, rotation], only analyze horizontal sweep angle
y = wingAngles(:,2);
x = wingAngles(:,1)/500;
genPlots = createFigures;
%[strokeSummaryData, strokeDataOverSegment, strokeSummaryStruct] = getWingstrokeSummaryData(x, y, numHCLsPerFit, frameRate, genPlots, 'derivativeHCL');
[strokeSummaryData, strokeDataOverSegment, strokeSummaryStruct] = getWingstrokeSummaryDataViaSpline(x, y, frameRate, genPlots);


tds.rawWingSweepAngle = wingAngles(:,[1 2]);
tds.rawWingElevationAngle = wingAngles(:,[1 3]);
tds.rawWingRotationAngle = wingAngles(:,[1 4]);
tds.strokeSummaryStruct = strokeSummaryStruct;
tds.strokeSummaryData = strokeSummaryData;
tds.strokeDataOverSegment = strokeDataOverSegment;

infoStructEntry.trialData = tds;

