function infoStructEntry = calculateWingKinematicsInBody(infoStructEntry)
%calculateWingKinematicsinBody - calculates wing kinematics with respect to the
% body frontal plane (body frame).
% Stroke sweep/positional angle, stroke deviation/elevation angle, and wing
% rotation angle (about its axis) are calculated for each frame.  For each
% stroke we caluclate stroke amplitude and offset, stroke deviation 
% amplitude and stroke frequency.  Many trials do not have defined stroke
% patterns, so we also calculate wing holding, defined as the percentage of
% time the wings were held both backward and elevated.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing wing kinematics data
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

%%
% [themin, minidx] = min(arrayfun(@(x) x.lastRealFrame-x.firstRealFrame, trialInfoStruct))
% [themax, maxidx] = max(arrayfun(@(x) x.lastRealFrame-x.firstRealFrame, trialInfoStruct))
%infoStructEntry = trialInfoStruct(6);

tds = infoStructEntry.trialData;

% If right wing was tracked, reflect about the body axis so angles will
% work out. Note, only one wing (closest to side camera) per transit was tracked.
% Don't need to do this if x coordinate was reflected in
% loadAndProcessTrial.m.
switchRL = false; % strcmp(infoStructEntry.flightDirection,'dB');

tds.tFromWBToFF4rawPInBody = tds.tFromBBToFF4rawPInBody-tds.tFromBBToWBsmoothedPInBody;
    
% initialize the wing angle matrix [frameNums, sweep, deviation, rotation]
wingAngles = [tds.frameNum,...
    NaN(size(tds.frameNum,1),3)];

% tempStuff = [tds.frameNum,...
%     NaN(size(tds.frameNum,1),5)];

for frameNumber = 1:length(tds.frameNum)
    tFromWBToWTInBody = tds.tFromWBToWTrawPInBody(frameNumber,2:4);
    tFromWBToFF4InBody = tds.tFromWBToFF4rawPInBody(frameNumber,2:4);
    
    if switchRL
        tFromWBToWTInBody(2) = -tFromWBToWTInBody(2);
        tFromWBToFF4InBody(2) = -tFromWBToFF4InBody(2);
    end
    
    %tempStuff(frameNumber,2:4) = tFromWBToWTInBody;
    
    if sum(isnan(tFromWBToWTInBody))>0
        wingAngles(frameNumber, 2:4) = NaN(1,3);
    else
        
        % 1. Rotate all wing points to wingSW frame
        swRotVec = vrrotvec([0 1 0], [tFromWBToWTInBody(1:2),0]);
        RFromBodyToWingSW = vrrotvec2mat(swRotVec);
        tFromWBToWTinWingSW = tFromWBToWTInBody*RFromBodyToWingSW;
        tFromWBToFF4inWingSW = tFromWBToFF4InBody*RFromBodyToWingSW;
        
        % 2. Rotate all wing points to wingEL frame
        elRotVec = vrrotvec([0 1 0], tFromWBToWTinWingSW);
        RFromWingSWToWingEL = vrrotvec2mat(elRotVec);
        tFromWBToWTinWingEL = tFromWBToWTinWingSW*RFromWingSWToWingEL;
        tFromWBToFF4inWingEL = tFromWBToFF4inWingSW*RFromWingSWToWingEL;
        
        swelRotVec = vrrotvec([0 1 0], tFromWBToWTInBody);
        RFromBodyToWingEL = vrrotvec2mat(swelRotVec);
        tFromWBToWTinWingEL2 = tFromWBToWTInBody*RFromBodyToWingEL;
        tFromWBToFF4inWingEL2 = tFromWBToFF4InBody*RFromBodyToWingEL;
        
        % 3. Rotate all wing points to wing frame
        % Want to rotate the normal of the wing plane so that it points in
        % the +z direction.
        if sum(isnan(tFromWBToFF4InBody))>0
            roRotVec = [NaN, NaN, NaN, NaN];
        else
            plNorm = cross(tFromWBToWTinWingEL, tFromWBToFF4inWingEL);
            roRotVec = vrrotvec([0 0 1], plNorm);
            RFromWingELToWing = vrrotvec2mat(roRotVec);
            tFromWBToWTinWing = tFromWBToWTinWingEL*RFromWingELToWing;
            tFromWBToFF4inWing = tFromWBToFF4inWingEL*RFromWingELToWing;
        end
        
        wingAngles(frameNumber,2:4) = [...
            -swRotVec(4)*sign(swRotVec(3)),...
            elRotVec(4)*sign(elRotVec(1)),...
            -roRotVec(4)*sign(roRotVec(2))]*180/pi;
        
%         tempStuff(frameNumber,5) = -swRotVec(4)*sign(swRotVec(3));
%         tempStuff(frameNumber,6) = elRotVec(4)*sign(elRotVec(1));
%         
%         k = frameNumber;
%         subplot(1,3,1)
%         plot(tempStuff(:,1), tempStuff(:,5))
%         hold on
%         plot(tempStuff(k,1), tempStuff(k,5),'ro')
%         hold off
%         subplot(1,3,2)
%         plot(tempStuff(:,1), tempStuff(:,6))
%         hold on
%         plot(tempStuff(k,1), tempStuff(k,6),'ro')
%         hold off
%         subplot(1,3,3)
%         plot3(tempStuff(:,2),tempStuff(:,3),tempStuff(:,4))
%         hold on
%         plot3(tempStuff(k,2),tempStuff(k,3),tempStuff(k,4),'ro')
%         hold off
%         pbaspect([1 1 1]);
%         daspect([1 1 1]);
%         pause(0.125);
%         drawnow limitrate
        
        % "Angle of attack" (ignoring that the wing is upside down)
        % if wingAngles(frameNumber,4) > 90
        %    wingAngles(frameNumber,4) = 180-wingAngles(frameNumber,4);
        % end
         
        % can also get the wing sweep and elevation like this:
        % [wingH,wingE,~] = cart2sph(tFromWBToWTInBody(1),tFromWBToWTInBody(2),tFromWBToWTInBody(3));
        % wingAngles(frameNumber,2:3) = [-wingH+pi/2, wingE]*180/pi;
        
    end
end


%% Calculate wing stroke amplitude, frequency and offset for sweep angle
% Calculate trajectories and also means for various timing segments

tds.rawWingSweepAngleInBody = wingAngles(:,[1 2]);
tds.rawWingElevationAngleInBody = wingAngles(:,[1 3]);
tds.rawWingRotationAngleInBody = wingAngles(:,[1 4]);

infoStructEntry.trialData = tds;

