function [wingPhase, wingFrequency, wingAngle] = getWingstrokePhaseAtBillBaseEntry(x, y, frameRate, genPlots)
%getWingstrokePhaseAtBillBaseEntry - calculates wing stroke phase at the
%time of gust entry (when bill base crosses the midline)
%
% Inputs:
%    x, y - 1D vectors x as real times WITH x = 0 when the bill base crosses
%           the gust midline, y as wing sweep angles in degrees
%    frameRate - the number of samples per second
%
% Outputs:
%    wingPhase - in radians
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018


% clear the NaN rows
nanRows = isnan(y);

x = x(~nanRows);
y = y(~nanRows);

% OPTION USING DERAVITIVES
% First take numerical derivative to find highs/lows and crossings
dy = NaN(size(y,1),1);
dy(2:end-1,:) = (y(3:end,:)-y(1:end-2,:))*frameRate/2;
highlowXs = x(dy .* circshift(dy,1) <= 0);
crossingXs = (highlowXs(1:end-1) + highlowXs(2:end))/2;

% Store the points of interest labeled as 1 if a high or low and 0 if a
% crossing
hcls = sortrows([highlowXs ones(size(highlowXs)); ...
    crossingXs zeros(size(crossingXs))]);
per = mean(diff(hcls(:,1)))*4; 
if isnan(per)
    per = 40; % if we don't get a guess for period, guess 40 Hz.
end

sinFits = ones(1,8); % [time, amplitude, frequency, phase, offset, high, low, fitQuality]

yr = max(y) - min(y);
yoff = (max(y)+min(y))/2;
fit = @(b,x)  b(1).*(sin(2*pi*x./b(2) + 2*pi/b(3))) + b(4);    % Function to fit
fcn = @(b) sum((fit(b,x) - y).^2);             % Least-Squares cost function
%fcn = @(b) sum(fitWeights.*(fit(b,xsel) - ysel).^2);  % Least-Squares cost function with weights
phases = -1:0.25:1;
ss = ones(length(phases),4);
errors = ones(1,length(phases));
for i = 1:length(phases)
    %[s2, ~, exitflag] = fminsearch(fcn, [yr/2;  per;  phases(i);  fnval(spl, xmean)]);
    options = optimset('Display','off');
    [s2, ~, exitflag] = fminsearch(fcn, [yr/2;  per;  phases(i);  yoff], options);
    if ~exitflag
        %disp('Max iterations reached, increasing to 400*numvariables');
        options = optimset('MaxFunEvals',400*4,'Display','off');
        %s2 = fminsearch(fcn, [yr/2;  per;  phases(i);   fnval(spl, xmean)], options);
        s2 = fminsearch(fcn, [yr/2;  per;  phases(i);  yoff], options);
    end
    ss(i,:) = s2;
    errors(i) = fcn(ss(i,:));
end
[sMin, sMinIdx] = min(errors);
s2 = ss(sMinIdx,:)';

% shiftPhase((2*pi*(-0.026)./s2(2)+2*pi/s2(3))*180/pi)

sinFits(1) = shiftPhase(2*pi/s2(3)*180/pi); %shift phases so between -180 and 180 deg
sinFits(2:5) = s2';
sinFits(3) = 1/sinFits(3); % convert from period to frequency
sinFits(6:7) = [s2(4)+s2(1), s2(4)-s2(1)];
sinFits(8) = sMin/length(x);

if genPlots
    figure()
    subplot(2,1,1)
    plot(x,y,'black')
    hold on
    plot(crossingXs, zeros(length(crossingXs),1),'ro')
    plot(highlowXs, zeros(length(highlowXs),1),'go')
    tResamp = min(x):0.0005:0;
    plot(tResamp,fit(s2,tResamp),'b.')
    axis([min(x), 0, -inf, inf])
    subplot(2,1,2)
    plot(x,dy)
    axis([min(x), 0, -inf, inf])
end

wingPhase = sinFits(1);
wingFrequency = sinFits(3);
wingAngle = fit(s2,0);
