function [strokeSummaryData, strokeDataOverSegment, strokeSummaryStruct] = getWingstrokeSummaryData(x, y, numHCLsPerFit, frameRate, genPlots, fitmode)
%getWingstrokeSummaryData - calculates wing stroke amplitude, frequency,
%offset (the angle about which flapping occurs), min and max positional
%angles as functions of time.  Also reports means, standard deviations,
%standard errors, and 5-95 percentile ranges over the entire trial.
%
% Inputs:
%    x, y - 1D vectors x = real times, y = wing angles in degrees
%    numHCLsPerFit - number of highs, crossings, and lows before and after
%       each point to include in segment fits.  A sine fit is performed on
%       each segment of data in sliding window fashion.
%    frameRate - the number of samples per second
%
% Outputs:
%    strokeSummaryData - a matrix with rows consisting of:
%       {'time','amplitude', 'frequency', 'phase', 'offset', ...
%       'maxPosAng','minPosAng','fitError'} and columns
%       {'mean', 'std', 'SE','5-95 percentile range'}
%    strokeDataOverSegment - a matrix containing (columns)
%       {'time','amplitude', 'frequency', 'phase', 'offset', ...
%       'maxPosAng','minPosAng','fitError'} over time where each row
%       corresponds to the middle of the time segment for each fit.
%    strokeSummaryStruct - a struct to keep everything organized
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018


% clear the NaN rows
nanRows = isnan(y);

x = x(~nanRows);
y = y(~nanRows);

% OPTION USING SPLINE AND DETECTING CROSSINGS

% maybe do a spline fit and then add a term to the fit function that blows
% up if the amplitude coefficient is greater than the max-min for the
% entire trial.

if strcmp(fitmode, 'splineCrossings')
    ymf = movmean(y,40);
    %ymf = movmean(ymf,81);

    numKnots = 5;
    spl = spap2(numKnots,5,x,ymf);
    ysm = fnval(spl, x);

    if genPlots
        figure()
        subplot(1,2,1)
        plot(x*500, y, x*500, ysm)
        subplot(1,2,2)
        plot(x*500, y-ysm)
    end

    yz = y-ysm;
    crossIdxs = find(yz .* circshift(yz,1) <= 0);
    zx = x(crossIdxs);     % Find zero-crossings
    zx(1) = [];
    per = 2*mean(diff(zx));                     % Estimate period
    hcls = sortrows([zx zeros(size(zx))]);
    numHCLsPerFit = 1;
end

% OPTION USING DERAVITIVES
if strcmp(fitmode, 'derivativeHCL')
    % First take numerical derivative to find highs/lows and crossings
    dy = NaN(size(y,1),1);
    dy(2:end-1,:) = (y(3:end,:)-y(1:end-2,:))*frameRate/2;
    highlowXs = x(dy .* circshift(dy,1) <= 0);
    crossingXs = (highlowXs(1:end-1) + highlowXs(2:end))/2;
    per = 2*mean(diff(crossingXs)); 

    % Store the points of interest labeled as 1 if a high or low and 0 if a
    % crossing
    hcls = sortrows([highlowXs ones(size(highlowXs)); ...
        crossingXs zeros(size(crossingXs))]);
    
    if genPlots
        subplot(2,1,1)
        plot(x,y)
        hold on
        plot(crossingXs, zeros(length(crossingXs),1),'ro')
        plot(highlowXs, zeros(length(highlowXs),1),'go')
        axis([min(x), max(x), -inf, inf])
        subplot(2,1,2)
        plot(x,dy)
        axis([min(x), max(x), -inf, inf])
    end
end

% OPTOIN USING PEAK FINDING
% [pks,plocs] = findpeaks(y-ysm,x, 'MinPeakProminence', 10, 'MinPeakDistance', 1/60);
% [lws,llocs] = findpeaks(-(y-ysm),x, 'MinPeakProminence', 10, 'MinPeakDistance', 1/60);
% lws = -lws;
% 
% plot(x,y-ysm,plocs,pks,'o',llocs,lws,'o')
% 
% zx = locs;
% per = mean(diff(zx));

% if genPlots
%     figure(2)
%     plot(x,y)
%     hold on
%     plot(zx,ysm(crossIdxs(2:end)),'ro')
%     hold off
% end

% 
% crossNum = 1;
% numPeaksPerFit = 3;
% ysel = y(x>=zx(crossNum) & x<zx(crossNum+numPeaksPerFit));
% xsel = x(x>=zx(crossNum) & x<zx(crossNum+numPeaksPerFit));
% plot(x,y)
% hold on
% plot(x,y,'bo')
% plot(xsel,ysel,'r.')
% plot(hcls(:,1),zeros(length(hcls),1),'go')
% hold off

%%% A sin fit for each high/crossing/low (hcl)
% If we only have a few hcls, use the maximum number of peaks
% numHCLsPerFit is the number of samples before and after each point of interest
if length(hcls) < numHCLsPerFit*2+1
    numHCLsPerFit = ceil(length(hcls)/2)-1;
end
% we will need to know whether numHCLs is even or odd
numHCLsOdd = mod(numHCLsPerFit,2);

if genPlots
    f1 = figure('units','normalized','outerposition',[0 0 1 1]);
end
sinFits = ones(length(hcls)-2*numHCLsPerFit,8); % [time, amplitude, frequency, phase, offset, high, low, fitQuality]
for hclNum = 1:(length(hcls)-2*numHCLsPerFit)
    
%     % If we're looking at a high or low, and numHCLs is odd, use
%     % numHCLsPerFit-1 previous and future crossings:
%     if hcls(hclNum+numHCLsPerFit,2) == 1 % it's a high or low
%         % collect two previous crossings/peaks and to next crossings/peaks
%         % If numHCLs is odd, these will be crossings: hcls(:,2) == numHCLsOdd
%         %             and these are crossings before: hcls(:,1) < hcls(hclNum+numHCLsPerFit,1)
%         bfOpts = hcls(hcls(:,2) == numHCLsOdd & hcls(:,1) < hcls(hclNum+numHCLsPerFit,1));
%         afOpts = hcls(hcls(:,2) == numHCLsOdd & hcls(:,1) > hcls(hclNum+numHCLsPerFit,1));
%         
%         % Select only numHCLsPerFit-1 most recent and future points:
%         bfOpts = bfOpts((end-(numHCLsPerFit-1)+1):end);
%         afOpts = afOpts(1:numHCLsPerFit-1);
%     
%     % Otherwise, we use numHCLsPerFit-1 previous and future highs and lows
%     else % it's a crossing
%         % collect two previous peaks/crossings and to next peaks/crossings
%         % If numHCLs is odd, these will be peaks: hcls(:,2) == ~numHCLsOdd
%         %             and these are peaks before: hcls(:,1) < hcls(hclNum+numHCLsPerFit,1)
%         bfOpts = hcls(hcls(:,2) == ~numHCLsOdd & hcls(:,1) < hcls(hclNum+numHCLsPerFit,1));
%         afOpts = hcls(hcls(:,2) == ~numHCLsOdd & hcls(:,1) > hcls(hclNum+numHCLsPerFit,1));
%         
%         % Select only numHCLsPerFit-1 most recent and future points:
%         bfOpts = bfOpts((end-(numHCLsPerFit-1)+1):end);
%         afOpts = afOpts(1:numHCLsPerFit-1);
%     end
%     ysel = y(x >= bfOpts(1) & x <= afOpts(end));
%     xsel = x(x >= bfOpts(1) & x <= afOpts(end));
    
    
    ysel = y(x >= hcls(hclNum,1) & x <= hcls(hclNum+2*numHCLsPerFit,1));
    xsel = x(x >= hcls(hclNum,1) & x <= hcls(hclNum+2*numHCLsPerFit,1));
    
    yr = max(ysel) - min(ysel);
    yoff = (max(ysel)+min(ysel))/2;
    xmean = mean(xsel);
%     yselNorm = ysel-(max(ysel)+min(ysel))/2;
%     fitWeights = abs(yselNorm)/max(yselNorm); % weights
%     fitWeights(abs(yselNorm)<0.75*max(abs(yselNorm))) = 0.25*max(fitWeights); % emphasize points that are near the extremes (clicked more accurately)
%     fitWeights = fitWeights/sum(fitWeights); % normalize the weights
    fit = @(b,x)  sqrt(b(1).*b(1)).*(sin(2*pi*x./b(2) + 2*pi/b(3))) + b(4);    % Function to fit
    fcn = @(b) sum((fit(b,xsel) - ysel).^2);             % Least-Squares cost function
    %fcn = @(b) sum(fitWeights.*(fit(b,xsel) - ysel).^2);  % Least-Squares cost function with weights
    phases = -1:0.25:1;
    ss = ones(length(phases),4);
    errors = ones(1,length(phases));
    for i = 1:length(phases)
        %[s2, ~, exitflag] = fminsearch(fcn, [yr/2;  per;  phases(i);  fnval(spl, xmean)]);
        options = optimset('Display','off');
        [s2, ~, exitflag] = fminsearch(fcn, [yr/2;  per;  phases(i);  yoff], options);
        if ~exitflag
            %disp('Max iterations reached, increasing to 400*numvariables');
            options = optimset('MaxFunEvals',400*4,'Display','off');
            %s2 = fminsearch(fcn, [yr/2;  per;  phases(i);   fnval(spl, xmean)], options);
            s2 = fminsearch(fcn, [yr/2;  per;  phases(i);  yoff], options);
        end
        ss(i,:) = s2;
        errors(i) = fcn(ss(i,:));
    end
    [sMin, sMinIdx] = min(errors);
    s2 = ss(sMinIdx,:)';
%     [s2, ~, exitflag] = fminsearch(fcn, [yr;  per;  s(3);  ym]);
%     if ~exitflag
%         disp('Max iterations reached, increasing to 300*numvariables');
%         options = optimset('MaxFunEvals',400*4);
%         s2 = fminsearch(fcn, [yr;  per;  -1;  ym],options);
%     end
    sinFits(hclNum,1) = xmean;
    sinFits(hclNum,2:5) = s2';
    sinFits(hclNum,2) = abs(sinFits(hclNum,2));
    sinFits(hclNum,3) = 1/sinFits(hclNum,3); % convert from period to frequency
    sinFits(hclNum,6:7) = [s2(4)+s2(1), s2(4)-s2(1)];
    sinFits(hclNum,8) = sMin/length(xsel);

    if genPlots
%         figure(3)
%         xp = linspace(min(xsel),max(xsel),10*length(xsel));
%         plot(x,y,'b',  xp,fit(s,xp), 'r')
%         grid
%         drawnow limitrate
        
        figure(f1)
        subplot(2,1,1)
            hold on
            xp = linspace(min(xsel),max(xsel),10*length(xsel));
            plot(x,y,'b',  xp,fit(s2,xp), 'black')
            grid
            hold off
        subplot(2,1,2)
            xp = linspace(min(xsel),max(xsel),20*length(xsel));
            plot(xsel,ysel,'b',  xp,fit(s2,xp), 'r')
            grid
        drawnow limitrate
    end
    
end

if genPlots
    figure(4)
    figTitles = {'amplitude', 'frequency', 'phase', 'offset','highs','lows','meanFitError'};
    for i = 1:size(sinFits,2)-1
        subplot(3,3,i)
        plot(sinFits(:,1), sinFits(:,i+1))
        axis([min(x) max(x) -inf inf])
        title(figTitles{i})
    end
    subplot(3,3,4)
    %hold on
    %plot(x, ysm)
    %hold off
    subplot(3,3,8)
    plot(x, y)
    axis([min(x) max(x) -inf inf])
end

% Sometimes we get a bad fit when the wings are paused for long periods
if strcmp(fitmode, 'derivativeHCL') && (sum(abs(sinFits(:,5)) > 100)>0 || sum(sinFits(:,6)>120)>0 || sum(sinFits(:,7)<-120)>0)
    
    [strokeSummaryData, strokeDataOverSegment, strokeSummaryStruct] = getWingstrokeSummaryData(x, y, 1, frameRate, genPlots, 'splineCrossings');
    
else

    strokeDataOverSegment = sinFits;
    if size(sinFits,1) > 1
        strokeSummaryData = [mean(sinFits)', std(sinFits)', std(sinFits)'./sqrt(size(sinFits,1)), (prctile(sinFits,95)-prctile(sinFits,5))'];
    else
        strokeSummaryData = [sinFits', NaN(size(sinFits,2)), NaN(size(sinFits,2)), NaN(size(sinFits,2))];
    end
    dataNames = {'time','amplitude', 'frequency', 'phase', 'offset','maxPosAng','minPosAng','fitError'};
    strokeSummaryStruct = struct();
    strokeSummaryStruct.columnInfo = ['mean,std,stdErr,range: calculated for several fits '...
        'over 1.5 period intervals capturing 3 troughs/peaks each.'];
    columnNames = {'mean','std','stdErr','FiveToNinetyFivePercentileRange'};
    for i = 1:length(dataNames)-1
        colStruct = struct();
        for j = 1:length(columnNames)
            colStruct.(columnNames{j}) = strokeSummaryData(i+1,j);
        end
        colStruct.data = [strokeDataOverSegment(:,1), strokeDataOverSegment(:,i+1)];
        strokeSummaryStruct.(dataNames{i+1}) = colStruct;
    end

end
