function infoStructEntry = loadAndProcessTrial(infoStructEntry, gustPositionStruct, dataStartFrame, dataEndFrame, smoothTolerance, createFigures)
%loadAndProcessTrial - loads manually tracked data from csv file, rotates data
%from calibration frame to lab frame, translates data so that origin is at
%the center of the air gust, fits smoothing splines to position data
%
% Inputs:
%    filename - filename of data file
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    dataStartFrame - first frame number for which to keep data (inclusive)
%    dataEndFrame - last frame number for which to keep data (inclusive)
%                 - use string 'end' to specify end of data
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing rotated, translated, and
%    smoothed trajectory and kinematics data
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

% infoStructEntry = trialInfoStruct(23); dataStartFrame = 1; dataEndFrame = 'end';

fdIdx = strfind(infoStructEntry.filename,'_d');
infoStructEntry.flightDirection = infoStructEntry.filename(fdIdx(1)+1:fdIdx(1)+2);
gtIdx = strfind(infoStructEntry.filename,'_p');
infoStructEntry.gustTreatment = infoStructEntry.filename(gtIdx(1)+2:gtIdx(1)+6);
tnIdx = strfind(infoStructEntry.filename,'_');
infoStructEntry.trialNumber = str2num(infoStructEntry.filename(tnIdx(4)-3:tnIdx(4)-1));

[~, ~, raw] = xlsread(fullfile(infoStructEntry.folder, infoStructEntry.filename));

firstDataRowIndex = 2;
rawData = strNaNs2numNaNs(raw(firstDataRowIndex:end,:));

% Used to select a specific data start frame number (e.g. when a bird
% briefly entered the gust, backed out, then went all the way through, we
% only want the portion where the bird is going all the way through).
dataStartFrame = max(dataStartFrame,1);
if strcmp(dataEndFrame,'end')
    dataEndFrame = size(rawData,1);
else
    dataEndFrame = min(dataEndFrame,size(rawData,1));
end
rawData(1:dataStartFrame-1,:) = NaN(size(rawData(1:dataStartFrame-1,:)));
rawData(dataEndFrame+1:end,:) = NaN(size(rawData(dataEndFrame+1:end,:)));

nonNaNFrameNums = find(sum(~isnan(rawData(:,4:6)),2)==3 & sum(~isnan(rawData(:,16:18)),2)==3);
firstRealFrame = nonNaNFrameNums(1);
lastRealFrame = nonNaNFrameNums(end);

infoStructEntry.firstRealFrame = firstRealFrame;
infoStructEntry.lastRealFrame = lastRealFrame;

frameNums = (1:size(rawData,1))';

correspBirdNames = arrayfun(@(x) x.birdName, gustPositionStruct, 'UniformOutput',false);
gustPosIdx = find(arrayfun(@(x) strcmp(x, infoStructEntry.birdID), correspBirdNames));
tFromOToANCinCD = gustPositionStruct(gustPosIdx).tFromOToANCinCD;
RFromCDToAN = gustPositionStruct(gustPosIdx).RFromCDToAN;

numPts = size(rawData,2)/3;
numSamples = size(rawData,1);
allPtsColumn = NaN(numPts*numSamples,3);
for ptNum = 1:numPts
    allPtsColumn((ptNum-1)*numSamples+1:ptNum*numSamples,:) = rawData(:,(ptNum-1)*3+1:ptNum*3);
end

% Translate
tFromOToXYZinCD = allPtsColumn;
tFromANCToXYZinCD = -tFromOToANCinCD + tFromOToXYZinCD;
% Rotate
tFromANCToXYZinAN = tFromANCToXYZinCD * RFromCDToAN;

ptsInAirKnifeFrame = NaN(size(rawData));
for ptNum = 1:numPts
    ptsInAirKnifeFrame(:,(ptNum-1)*3+1:ptNum*3) = tFromANCToXYZinAN((ptNum-1)*numSamples+1:ptNum*numSamples,:);
end

rFromOToGPinCD = gustPositionStruct(gustPosIdx).pointPositions([1:8],:);
rFromANCToGPinCD = -tFromOToANCinCD + rFromOToGPinCD;
rFromANCToGPinAN = rFromANCToGPinCD * RFromCDToAN;

if createFigures
    figure()
    hold on
    for ptNum = 1:numPts
        thePts = ptsInAirKnifeFrame(:,(ptNum-1)*3+1:ptNum*3);
        plot3(thePts(:,1), thePts(:,2), thePts(:,3))
        xlabel('x')
        ylabel('y')
        zlabel('z')
        pbaspect([1 1 1]);
        daspect([1 1 1]);
    end
    plot3(rFromANCToGPinAN(:,1),rFromANCToGPinAN(:,2),rFromANCToGPinAN(:,3),'ro');
    hold off
end

% Negate the x coordinate (along the tunnel axis) to make all trials go the
% same direction (we will also keep track of the actual direction, of course).
switchFD = strcmp(infoStructEntry.flightDirection,'dB');
if switchFD
    ptsInAirKnifeFrame(:,1:3:30) = -ptsInAirKnifeFrame(:,1:3:30);
end

%% Smoothing head, abdomen, and wingbase trajectories
tds = struct('frameNum',frameNums);

tds.tFromANCToBBrawPInLab = [frameNums ptsInAirKnifeFrame(:,2*3-2:2*3)];
tds.tFromANCToRUrawPInLab = [frameNums ptsInAirKnifeFrame(:,6*3-2:6*3)];
tds.tFromANCToWTrawPInLab = [frameNums ptsInAirKnifeFrame(:,7*3-2:7*3)];
tds.tFromANCToWBrawPInLab = [frameNums ptsInAirKnifeFrame(:,10*3-2:10*3)];
tds.tFromANCToTCrawPInLab = [frameNums ptsInAirKnifeFrame(:,5*3-2:5*3)];
tds.tFromANCToTTrawPInLab = [frameNums ptsInAirKnifeFrame(:,4*3-2:4*3)];
tds.tFromANCToFF4rawPInLab = [frameNums (ptsInAirKnifeFrame(:,8*3-2:8*3)+ptsInAirKnifeFrame(:,9*3-2:9*3))./2];

% This gives the frameNumber just before each zero crossing
tds.zeroCrossingFrameNums = find(abs(diff(sign(tds.tFromANCToBBrawPInLab(:,2))))>0.5);

tdsFields = fields(tds);
trackedFeatureIdxs = cellfun(@(x) contains(x,'InLab'), tdsFields);
tdsFields = tdsFields(trackedFeatureIdxs);
smoothIndices = cellfun(@(x) ~(contains(x,'WT') | contains(x,'FF')),tdsFields);
featureNamesToSmooth = tdsFields(smoothIndices);

smoothedFeatureNames = cellfun(@(x)...
    strrep(x,'rawP','smoothedPVA'), featureNamesToSmooth,'UniformOutput',false);

splineFeatureNames = cellfun(@(x)...
    strrep(x,'rawP','spline'), featureNamesToSmooth,'UniformOutput',false);

for featureNum = 1:length(featureNamesToSmooth)
    t = (firstRealFrame:lastRealFrame)';
    pointOfInterest = tds.(featureNamesToSmooth{featureNum});
    pointOfInterest = [t pointOfInterest(firstRealFrame:lastRealFrame,2:4)];
    tInterpolate = (firstRealFrame:lastRealFrame)';
    weights = ones(1, size(t,1));
    thingPoss = [tds.frameNum, NaN(size(tds.frameNum,1), 3)];
    thingVels = [tds.frameNum, NaN(size(tds.frameNum,1), 3)];
    thingAccs = [tds.frameNum, NaN(size(tds.frameNum,1), 3)];

    % Create a smoothing spline
    tolerance = smoothTolerance; % Hand-tuned to match trajectories with reasonable acceleration
    splNames = {'x','y','z'};
    for i = 1:3
        % spaps(times, cordLocations, tolerance, weights, 3 = quintic)
        spl = spaps(t, pointOfInterest(:,i+1)', tolerance, weights, 3);
        spv = fnder(spl, 1);
        spa = fnder(spl, 2);
        thingPoss(firstRealFrame:lastRealFrame,i+1) = fnval(spl, tInterpolate);
        thingVels(firstRealFrame:lastRealFrame,i+1) = fnval(spv, tInterpolate);
        thingAccs(firstRealFrame:lastRealFrame,i+1) = fnval(spa, tInterpolate);
        
        tds.(splineFeatureNames{featureNum}).(['p' splNames{i}])  = spl;
        tds.(splineFeatureNames{featureNum}).(['v' splNames{i}])  = spv;
        tds.(splineFeatureNames{featureNum}).(['a' splNames{i}])  = spa;
    end

    scaledPoss = [thingPoss(:,1), thingPoss(:,2:4)];
    scaledVels = [thingVels(:,1), thingVels(:,2:4)*(500/100)];
    scaledAccs = [thingAccs(:,1), thingAccs(:,2:4)*(500^2/100)];

    if createFigures
        plotColors = {'black','blue','red'};
        figure()
        for i = 2:4
            subplot(3,1,i-1)
            plot(thingPoss(:,1),thingPoss(:,i))
            hold on
            plot(pointOfInterest(:,1),pointOfInterest(:,i))
            hold off
        end

        figure()
        hold on
        for i=2:4
            plot(scaledVels(:,1),scaledVels(:,i), plotColors{i-1})
        end
        hold off
        
        figure()
        hold on
        for i=2:4
            plot(scaledAccs(:,1),scaledAccs(:,i), plotColors{i-1})
        end
        hold off
        
        figure()
        plot3(thingPoss(:,2),thingPoss(:,3),thingPoss(:,4))
        hold on
        plot3(pointOfInterest(:,2),pointOfInterest(:,3),pointOfInterest(:,4),'ro')
        hold off
        xlabel('x')
        ylabel('y')
        zlabel('z')
        pbaspect([1 1 1]);
        daspect([1 1 1]);
        
    end

    tds.(smoothedFeatureNames{featureNum}) = [scaledPoss(:,1),...
        scaledPoss(:,2:4),scaledVels(:,2:4),scaledAccs(:,2:4)];
end

infoStructEntry.zeroCrossingFrameNums = tds.zeroCrossingFrameNums;

infoStructEntry.trialData = tds;

end