% This script processes raw 3D point trajectories and outputs a data
% structure containing body, wing, and tail kinematics for the paper:
%
% "Avoiding topsy-turvy: how Anna's Hummingbirds (Calypte anna) fly through upward gusts"
%   Marc Badger, Hao Wang, Robert Dudley
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018


addpath(genpath('C:\Users\marcbadger.MBCOMP\Google Drive\Air Knife Manuscript_Gust_MB\MatlabCode'));
workingDirectory = 'C:\Users\marcbadger.MBCOMP\Google Drive\Air Knife Manuscript_Gust_MB\RawData\Digitized Data';
createFigures = false;
saveTrialDataStruct = true;
loadExistingTrialDataStruct = false;
frameRate = 500;
smoothTolerance = 3; % 1 leads to large oscillations in body angular acceleration, no qualitative difference between 5 and 3
numHCLsPerFit = 3; % Number of oscillations to use for calculating wing kinematics

% Import the air knife position data corresponding to the calibration for each bird
gustPositionStruct = prepCalibrations(workingDirectory, createFigures);

% Get a list of the trial filenames in our dataset
trackedPointsFilenames = dir(fullfile(workingDirectory,...
    '*\b*psi*xyzpts.csv'));

trackedPointsFilenames.name

filenames = arrayfun(@(x) x.name, trackedPointsFilenames, 'UniformOutput', false);
foldernames = arrayfun(@(x) x.folder, trackedPointsFilenames, 'UniformOutput', false);
birdIDs = arrayfun(@(x) x.name(1:2), trackedPointsFilenames, 'UniformOutput', false);

% Create a struct that will keep track of all data for each trial
trialInfoStruct = struct('filename',filenames,'folder', foldernames, 'birdID', birdIDs);

%% For inspecting a particular trial
%trialNum = find(arrayfun(@(x) strcmp(x.filename, infoByBird{2}(15).filename), trialInfoStruct))
%trialNum = find(arrayfun(@(x) strcmp(x.filename, 'b4_jUP_p18psi_t001_dB_xyzpts.csv'), trialInfoStruct))

%% Load data and rotate+translate into the lab frame
for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = loadAndProcessTrial(trialInfoStruct(trialNum), gustPositionStruct, 1, 'end', smoothTolerance, createFigures);
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialDataStruct_smTol' num2str(smoothTolerance) '_' dt]), 'trialInfoStruct');
end

%% Trim data for a few bad trials to the correct segment
if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialInfoStruct*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

multiCrossIdxs = find(cell2mat(arrayfun(@(x) size(x.zeroCrossingFrameNums,1), trialInfoStruct, 'UniformOutput',false))>1);
trialInfoStruct(multiCrossIdxs).filename
trialInfoStruct(multiCrossIdxs).zeroCrossingFrameNums

% Correct a trial in which a bird briefly entered the gust, backed out, 
% then went all the way through. We only want the portion where the bird
% is going all the way through.
multiCrossIdxs(1)
trialNum = 13;
dataStartFrame = trialInfoStruct(trialNum).zeroCrossingFrameNums(2)+10;
infoStructEntry = loadAndProcessTrial(trialInfoStruct(trialNum), gustPositionStruct, dataStartFrame, 'end', smoothTolerance, createFigures);

% Copy the new fields to the existing data structure
entryFields = fields(infoStructEntry);
for field = 1:length(entryFields)
    trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
end

% Fixed!
find(arrayfun(@(x) length(x.zeroCrossingFrameNums)>1,infoStructEntry))

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialDataStruct_smTol' num2str(smoothTolerance) '_' dt]), 'trialInfoStruct');
end

%% Calculate timings of transit phases
if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialDataStruct*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = calculateTransitPhaseTimings(trialInfoStruct(trialNum));
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialBirdZStruct_smTol' num2str(smoothTolerance) '_' dt]), 'trialInfoStruct');
end

%% Rotate data from the lab frame to the birdZ frame
% The birdZ frame is defined by birdZ_z = lab_z and birdZ_x = vector
% from rump to bill base projected onto lab_x-lab_y plane. Data are also
% translated so that the origin is at the bill base.

if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialDataStruct*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = rotateDataToBirdZFrame(trialInfoStruct(trialNum), createFigures);
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialBirdZStruct_smTol' num2str(smoothTolerance) '_' dt]), 'trialInfoStruct');
end

%% Rotate data from the birdZ frame to the body frame and calculate body kinematics
% The bird body frame is defined by body_x = vector from rump to bill
% base, body_y perpendicular to body_x with dot(body_y, lab_z) = 0, and
% body_z = cross(body_x, body_y).  In practice, the body frame is simply
% rotated about birdZ_y (pitched up/down) until body_x is aligned with the
% vector from rump to bill base.  We also calculate tail angles and the 
% body angular velocity and acceleration.

if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialBirdZStruct*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = rotateDataToBirdFrame(trialInfoStruct(trialNum), frameRate, createFigures);
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialBirdStruct_smTol' num2str(smoothTolerance) '_' dt]), 'trialInfoStruct');
end

%% CHECKING SMOOTHING

structOptions = dir(fullfile(workingDirectory,'trialBirdStruct*'));
structOptions.name
optStruct = cell(3,1);
for j = 1:length(structOptions)
    loadedStruct = load(fullfile(workingDirectory, structOptions(j).name));
    optStruct{j} = loadedStruct.trialInfoStruct;
end

figure()
for j = 1:length(optStruct)
    q95s = arrayfun(@(x) quantile(sqrt(sum(x.trialData.tFromANCToBBsmoothedPVAInLab(:,8:10).^2,2)),0.95),optStruct{j});
    subplot(3,1,j)
    hist(q95s)
    axis([0 12 0 30])
    title({['Effect of smoothing tolerance ' num2str(2*j+1) 'on 95 percentile of acceleration of bill base']})
end

plotNames = {'Body elevation angle (deg)',...
    'Body angular velocity about pitch (deg/s)',...
    'Tail elevation relative to horizontal (deg)',...
    'Tail elevation relative to body (deg)'};
for j = 1:length(optStruct)
    for i = 1:length(optStruct{j})
        tempPos = optStruct{j}(i).trialData.tFromANCToBBsmoothedPVAInLab;
        tempSWBZ = optStruct{j}(i).trialData.birdElevation;
        tempSWB = optStruct{j}(i).trialData.bodyAngularVelocityinBody;
        tempWEBZ = optStruct{j}(i).trialData.tailElevationInBirdZ;
        tempWEB = optStruct{j}(i).trialData.tailElevationInBody;
        frf = round(optStruct{j}(i).zeroCrossingFrameNums);
        lrf = optStruct{j}(i).lastRealFrame;
        tcmf = round(nanmin([optStruct{j}(i).tTCCrossMidline,optStruct{j}(i).tTCLeaveGust,lrf]));
        tlgf = round(nanmin([optStruct{j}(i).tTCLeaveGust,lrf]));
    
        subplot(4,3,j)
        plot(tempPos(:,2), tempSWBZ(:,2), 'Color', 'black');
        plot(tempPos(frf:tcmf,2), tempSWBZ(frf:tcmf,2), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempSWBZ(tcmf:tlgf,2), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempSWBZ(tlgf:lrf,2), 'Color', [0 0 1]);
        hold on
        axis([-20 20 -40 100])
        subplot(4,3,3+j)
        plot(tempPos(:,2), tempSWB(:,3), 'Color', 'black');
        plot(tempPos(frf:tcmf,2), tempSWB(frf:tcmf,3), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempSWB(tcmf:tlgf,3), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempSWB(tlgf:lrf,3), 'Color', [0 0 1]);
        hold on
        axis([-20 20 -1500 1500])
        subplot(4,3,6+j)
        plot(tempPos(:,2), tempWEBZ(:,2), 'Color', 'black');
        plot(tempPos(frf:tcmf,2), tempWEBZ(frf:tcmf,2), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempWEBZ(tcmf:tlgf,2), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempWEBZ(tlgf:lrf,2), 'Color', [0 0 1]);
        hold on
        axis([-20 20 -100 100])
        subplot(4,3,9+j)
        plot(tempPos(:,2), tempWEB(:,2), 'Color', 'black');
        plot(tempPos(frf:tcmf,2), tempWEB(frf:tcmf,2), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempWEB(tcmf:tlgf,2), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempWEB(tlgf:lrf,2), 'Color', [0 0 1]);
        hold on
        axis([-20 20 -100 100])
    end
end
for i = 1:length(plotNames)
    subplot(4,3,3*i-2)
    title(plotNames(i))
    subplot(4,3,3*i-1)
    title(plotNames(i))
    subplot(4,3,3*i)
    title(plotNames(i))
end


%% WING KINEMATICS
% We want wing stroke, deviation and rotation angles relative to the horizontal plane (Altshuler et al 2012, Cheng et al. 2016)
% " The stroke plane frame of reference was shifted by the stroke plane
% angle (BETA), and the elevation amplitude (OMEGA) and stroke amplitude (PHI)
% were calculated once per stroke"
% I think we should also compute these relative to the body (i.e. wing
% sweep and wing elevation).

if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialBirdStruct*smTol3*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

% Wing kinematics where sweep is measured within the horizontal plane and
% elevation/deviation is measured away from the orizontal plane
for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = calculateWingKinematics(trialInfoStruct(trialNum), numHCLsPerFit, frameRate, createFigures);
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

% Wing kinematics where sweep is measured within the body frontal plane and
% elevation/deviation is measured ventrally or dorsally away from this
% plane
for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = calculateWingKinematicsInBody(trialInfoStruct(trialNum));
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
end

% Calculate wing phase at gust entry
for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    
    timeBefore = 0.04; % seconds
    if trialNum == 13 % trial 13 had a bad fit, due to some weird wing flapping,
        % so we fit to a shorter time span
        timeBefore = 0.02;
    end
    
    tempCrossFN = trialInfoStruct(trialNum).zeroCrossingFrameNums;
    tempSweep = trialInfoStruct(trialNum).trialData.rawWingSweepAngle;
    % fit only to the 20 frames before the bill base crossed the gust midline
    tempSweepSel = tempSweep(tempSweep(:,1) >= tempCrossFN-timeBefore*frameRate & tempSweep(:,1) < tempCrossFN,:);
    tempSweepSel(:,1) = (tempSweepSel(:,1) - trialInfoStruct(trialNum).tBBCrossMidline)/frameRate;
    x = tempSweepSel(:,1);
    y = tempSweepSel(:,2);

    [wingPhase, wingFrequency, wingAngle] = getWingstrokePhaseAtBillBaseEntry(x, y, frameRate, createFigures);
    
    trialInfoStruct(trialNum).wingPhaseAtBBCrossMidline = wingPhase;
    trialInfoStruct(trialNum).wingFrequencyAtBBCrossMidline = wingFrequency;
    trialInfoStruct(trialNum).wingAngleAtBBCrossMidline = wingAngle;
    
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialBirdStruct_wWingKinem_' dt]), 'trialInfoStruct');
end

%% CALCULATE MEANS FOR TIMING SEGMENTS
% Need means of body and wing kinematics for each combination of timing segments
% And also values and times for various points of interest (e.g. minimum
% body angle).

if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialBirdStruct_wWingKinem_*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

units = struct('time', {'seconds'}, 'distance', {'centimeters'}, ...
    'speed', {'meters/second'}, 'acceleration', {'meters/second^2'}, ...
    'angle', {'degrees'}, 'rotationRate', {'degrees/second'}, ...
    'rotationalAcceleration', {'degrees/second^2'}, ...
    'phase', {'degrees'}, 'frequency', {'Hz'});

% Wing kinematics where sweep is measured within the horizontal plane and
% elevation/deviation is measured away from the orizontal plane
for trialNum = 1:length(trialInfoStruct)

    disp(trialNum);
    infoStructEntry = calculateKinematicsMeansForTimeSegments(trialInfoStruct(trialNum), frameRate);
    
    % Copy the new fields to the existing data structure
    entryFields = fields(infoStructEntry);
    for field = 1:length(entryFields)
        trialInfoStruct(trialNum).(entryFields{field}) = infoStructEntry.(entryFields{field});
    end
    
    trialInfoStruct(trialNum).units = units;
    
end

for i = 1:length(trialInfoStruct)
frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
lrf = trialInfoStruct(i).lastRealFrame;
tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
tempOD = trialInfoStruct(i).trialData.strokeSummaryStruct.frequency.data;
odPos = NaN(size(tempOD,1),3);
for j = 1:size(tempOD,1)
    odPos(j,[1 3]) = tempOD(j,:);
    tpIdx = find(abs(tempPos(:,1)-tempOD(j,1)*500) <= 0.7,1);
    odPos(j,2) = tempPos(tpIdx,2);
end
temp0 = odPos(tempOD(:,1)*500 < frf,[2,3]);
temp1 = odPos(tempOD(:,1)*500 >= frf-1 & tempOD(:,1)*500 < tcmf,[2,3]);
temp2 = odPos(tempOD(:,1)*500 >= tcmf-1 & tempOD(:,1)*500 < tlgf,[2,3]);
temp3 = odPos(tempOD(:,1)*500 >= tlgf-1 & tempOD(:,1)*500 <= lrf,[2,3]);

trialInfoStruct(i).numberOfFlapsAfBbCmBeTcLg = size([temp1;temp2],1)/2;
trialInfoStruct(i).meanWBFAfBbCmBeTcLg = mean([temp1(:,2);temp2(:,2)]);
trialInfoStruct(i).meanWBFBeBbCm = mean(temp0(:,2));
trialInfoStruct(i).meanWBFAfTcLg = mean(temp3(:,2));
end

if saveTrialDataStruct
    dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
    save(fullfile(workingDirectory,['trialBirdStruct_wWingKinemAndMeanTimings_' dt]), 'trialInfoStruct');
end

histogram(arrayfun(@(x) x.segmentMeans.rawWingSweepAngleInBirdZ.AfTcCmBeTcLg(3), trialInfoStruct),'BinWidth',5)

%% EXPOPRT DATA SPREADSHEET

if loadExistingTrialDataStruct
    structOptions = dir(fullfile(workingDirectory,'trialBirdStruct_wWingKinemAndMeanTimings_*'));
    [~,idx] = sort([structOptions.datenum]);
    loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
    trialInfoStruct = loadedStruct.trialInfoStruct;
end

structFieldsToExportA = {'filename','birdID','trialNumber','flightDirection','gustTreatment',...
    'firstRealFrame','lastRealFrame','zeroCrossingFrameNums', ...
    'tBBEnterGust',...
        'bodyAngleAtBBEnterGust', 'bodyAngularAccelerationInBodyAtBBEnterGust',...
    'tBBCrossMidline', ...
        'zBBAtBBCrossMidline','yBBAtBBCrossMidline', 'vxBBAtBBCrossMidline', 'vzBBAtBBCrossMidline',...
        'bodyAngleAtBBCrossMidline', 'bodyAngularAccelerationInBodyAtBBCrossMidline',...
        'wingPhaseAtBBCrossMidline', 'wingFrequencyAtBBCrossMidline', 'wingAngleAtBBCrossMidline', ...
    'tTCCrossMidline', ...
        'zBBATTCCrossMidline', 'vxBBATTCCrossMidline', 'vzBBATTCCrossMidline', 'bodyAngleATTCCrossMidline', ...
    'tTCLeaveGust', ...
        'zBBATTCLeaveGust', 'vxBBATTCLeaveGust', 'vzBBATTCLeaveGust', 'bodyAngleATTCLeaveGust', ...
    'minBodyAngle', 'tminBodyAngle', ...
    'minBodyAngularVelocity', 'tminBodyAngularVelocity', ...
    'minBodyAngularAcceleration', 'tminBodyAngularAcceleration',...
    'numberOfFlapsAfBbCmBeTcLg','meanWBFAfBbCmBeTcLg','meanWBFBeBbCm','meanWBFAfTcLg'};

exportStruct = struct('filename',{trialInfoStruct.filename});
for trialNum = 1:length(exportStruct)
    for fieldNum = 1:length(structFieldsToExportA)
        fieldEntry = trialInfoStruct(trialNum).(structFieldsToExportA{fieldNum});
        if ~isstruct(fieldEntry)
            exportStruct(trialNum).(structFieldsToExportA{fieldNum}) = fieldEntry;
        else
            exportStruct(trialNum).(structFieldsToExportA{fieldNum}) = fieldEntry.value;
            exportStruct(trialNum).([structFieldsToExportA{fieldNum} 'Segment']) = fieldEntry.inSegment;
            
        end
    end
end

% Add segmentMeans substruct entries
for trialNum = 1:length(exportStruct)
    sms = trialInfoStruct(trialNum).segmentMeans;
    smsFields = fieldnames(sms);
    smsFields(1:2)=[];
    
    for fieldNum = 1:length(smsFields)
        fieldEntry = sms.(smsFields{fieldNum});
        subFields = fieldnames(fieldEntry);
        for subFNum = 1:length(subFields)
            exportStruct(trialNum).([smsFields{fieldNum} 'M' subFields{subFNum}]) = fieldEntry.(subFields{subFNum})(3);
        end
    end
end

% Collapse the struct into one vector for each field
dataStructForR = struct();
expFieldNames = fieldnames(exportStruct);
for fieldNum = 1:length(expFieldNames)
    if ischar(exportStruct(1).(expFieldNames{fieldNum}))
        dataStructForR.(expFieldNames{fieldNum}) = {exportStruct.(expFieldNames{fieldNum})}';
    else
        dataStructForR.(expFieldNames{fieldNum}) = [exportStruct.(expFieldNames{fieldNum})]';
    end
end

dataStructForR.noMBAORAirOff = strcmp(dataStructForR.tminBodyAngleSegment,'AfTrEnd') | strcmp(dataStructForR.gustTreatment,'00psi');
dataStructForR.MBABeTcCm = strcmp(dataStructForR.tminBodyAngleSegment,'BeBbCm') | strcmp(dataStructForR.tminBodyAngleSegment,'AfBbCmBeTcCm');
dataStructForR.MBABeTcLg = strcmp(dataStructForR.tminBodyAngleSegment,'BeBbCm') | strcmp(dataStructForR.tminBodyAngleSegment,'AfBbCmBeTcCm') | ...
    strcmp(dataStructForR.tminBodyAngleSegment,'AfTcCmBeTcLg');
dataStructForR.MBAAfTrEnd = strcmp(dataStructForR.tminBodyAngleSegment,'AfTrEnd');
dataStructForR.airKnifeOn = strcmp(dataStructForR.gustTreatment,'18psi');
dataStructForR.durationtCm = dataStructForR.tTCCrossMidline;
dataStructForR.durationtLg = dataStructForR.tTCLeaveGust;
dataStructForR.deltaHeightCm = dataStructForR.zBBATTCCrossMidline-dataStructForR.zBBAtBBCrossMidline;
dataStructForR.deltaHeightLg = dataStructForR.zBBATTCLeaveGust-dataStructForR.zBBAtBBCrossMidline;

dt = datestr(now,'yyyy-mm-dd_HH-MM-SS');
save(fullfile(workingDirectory,['gustDataForR_' dt]), 'dataStructForR');


%% EXPLORE BODY KINEMATICS

structOptions = dir(fullfile(workingDirectory,'trialBirdStruct*smTol3*'));
[~,idx] = sort([structOptions.datenum]);
loadedStruct = load(fullfile(workingDirectory, structOptions(idx(end)).name));
trialInfoStruct = loadedStruct.trialInfoStruct;

figure(3)
histogram(arrayfun(@(x) x.bodyAngularAccelerationInBodyAtBBCrossMidline, trialInfoStruct), 'BinWidth', 15)

fig = figure(2);
plotNames = {'Body elevation angle (deg)',...
    'Body angular velocity about pitch (rad/s)',...
    'Body angular acceleration about pitch (rad/s^2)',...
    'Tail elevation relative to horizontal (deg)',...
    'Tail extension angle (deg)'};
featNamesToPlot = {'tFromANCToBBsmoothedPVAInLab','birdElevation', 'bodyAngularVelocityinBody','bodyAngularAccelerationinBody',...
    'tailElevationInBirdZ','tailExtensionAngle'};
for featNum = 1:length(featNamesToPlot)
    if contains(featNamesToPlot{featNum},'Angular')
        dataIdx = 3;
    elseif contains(featNamesToPlot{featNum},'PVA')
        dataIdx = 4;
    else
        dataIdx = 2;
    end
    ymin = nanmin(arrayfun(@(x) nanmin(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));
    ymax = nanmax(arrayfun(@(x) nanmax(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));

    for i = 1:length(trialInfoStruct)
        frf = round(trialInfoStruct(i).firstRealFrame);
        lrf = trialInfoStruct(i).lastRealFrame;

        bent = round(trialInfoStruct(i).tBBEnterGustFrameNum);
        bbcm = round(trialInfoStruct(i).zeroCrossingFrameNums);
        tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
        tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));

        tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
        tempData = trialInfoStruct(i).trialData.(featNamesToPlot{featNum});

        if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
            %if trialInfoStruct(i).bodyAngleAtBBCrossMidline< 32
            if trialInfoStruct(i).bodyAngularAccelerationInBodyAtBBCrossMidline < -98
                j = 1;
            else
                j = 2;
            end
        else
            j = 3;
        end
        if j < 3
            subplot(length(featNamesToPlot),3,3*(featNum-1)+1)
            plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            hold on
            axis([-20 20 ymin ymax])
            subplot(length(featNamesToPlot),3,3*(featNum-1)+2)
            plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            hold on
            axis([-20 20 ymin ymax])
        end
        subplot(length(featNamesToPlot),3,3*(featNum-1)+j)
        plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', 'black');
        plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [255,128,0]/255);
        plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0 0 1]);
        hold on
        axis([-20 20 ymin ymax])
    end
end
for i = 1:length(featNamesToPlot)
    subplot(length(featNamesToPlot),3,3*(i-1)+1)
    title(["Gust: pitch accel < -98 " featNamesToPlot(i)])
    subplot(length(featNamesToPlot),3,3*(i-1)+2)
    title(["Gust: pitch accel > -98 " featNamesToPlot(i)])
    subplot(length(featNamesToPlot),3,3*(i-1)+3)
    title(["Control: " featNamesToPlot(i)])
end

orient(fig,'landscape')
fig.PaperPositionMode = 'manual';
print(fig, 'output.pdf', '-bestfit', '-painters', '-dpdf')

techniqueClusts = load('C:\Users\marcbadger.MBCOMP\Google Drive\Air Knife Manuscript_Gust_MB\For Dryad\TechniqueClustersForMatlab.mat');
structFilenames = arrayfun(@(x) x.filename, trialInfoStruct, 'UniformOutput',false);
cellfun(@(x) find(strcmp(x, structFilenames)), techniqueClusts.filenameID) % They are in order
trsBackup;
for i = 1:length(trialInfoStruct)
    trialInfoStruct(i).technique = techniqueClusts.technique{i};
end

% CAN ALSO PLOT BY BIRD ID
%{'tFromANCToBBsmoothedPVAInLab','birdElevation', 'bodyAngularVelocityinBody'}
%{'bodyAngularAccelerationinBody', 'tailElevationInBirdZ','tailExtensionAngle'}
%{'rawWingSweepAngleInBody'}
fig = figure(6);
featNamesToPlot = {'tailElevationInBirdZ'}; 
plotLabelName = {'tail elevation angle rel. horiz. (deg)'};
for featNum = 1:length(featNamesToPlot)
    if contains(featNamesToPlot{featNum},'Angular')
        dataIdx = 3;
    elseif contains(featNamesToPlot{featNum},'PVA')
        dataIdx = 4; % For plotting ZB with tFromANCToBBsmoothedPVAInLab
        % dataIdx = 5; % For plotting VX with tFromANCToBBsmoothedPVAInLab
    else
        dataIdx = 2;
    end
    ymin = nanmin(arrayfun(@(x) nanmin(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));
    ymax = nanmax(arrayfun(@(x) nanmax(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));

    for i = 1:length(trialInfoStruct)
        frf = round(trialInfoStruct(i).firstRealFrame);
        lrf = trialInfoStruct(i).lastRealFrame;

        bent = round(trialInfoStruct(i).tBBEnterGustFrameNum);
        bbcm = round(trialInfoStruct(i).zeroCrossingFrameNums);
        tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
        tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));

        tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
        tempData = trialInfoStruct(i).trialData.(featNamesToPlot{featNum});

        if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
            %if trialInfoStruct(i).bodyAngleAtBBCrossMidline< 32
            %if trialInfoStruct(i).bodyAngularAccelerationInBodyAtBBCrossMidline < -98*180/pi
            if strcmp(trialInfoStruct(i).technique, 'TailDominated')
                j = 1;
            else
                j = 2;
            end
        else
            j = 3;
        end
        birdNum = str2num(trialInfoStruct(i).birdID(2));
        if birdNum >= 3
            birdNum = birdNum -1;
        end
        if j < 3
            for birdNumIdx = 1:4
                subplot(length(featNamesToPlot)*4,3,3*(birdNumIdx-1)+1)
                hold on
                plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                axis([-20 20 ymin ymax])
                subplot(length(featNamesToPlot)*4,3,3*(birdNumIdx-1)+2)
                hold on
                plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
                axis([-20 20 ymin ymax])
            end
        end
        subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+j)
        hold on
        plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', 'black');
        plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [255,128,0]/255);
        plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0 0 1]);
        axis([-20 20 ymin ymax])
    end
end
for i = 1:length(featNamesToPlot)
    for birdNum = 1:4
        subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+1)
        title({['Tail-dominated: bird' num2str(birdNum) ': '], plotLabelName{i}})
        subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+2)
        title({['Wings-dominated: bird' num2str(birdNum) ': '] plotLabelName{i}})
        subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+3)
        title({['Control: bird' num2str(birdNum) ': '] plotLabelName{i}})
    end
end

% for i = 1:length(featNamesToPlot)
%     for birdNum = 1:4
%         subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+1)
%         title({['Gust: pitch accel < -98, bird' num2str(birdNum) ': '], plotLabelName{i}})
%         subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+2)
%         title({['Gust: pitch accel > -98, bird' num2str(birdNum) ': '] plotLabelName{i}})
%         subplot(length(featNamesToPlot)*4,3,3*(birdNum-1)+3)
%         title({['Control: bird' num2str(birdNum) ': '] plotLabelName{i}})
%     end
% end

orient(fig,'landscape')
fig.PaperPositionMode = 'manual';
print(fig, ['tail elevation angle' '_byBirdXTechnique.pdf'], '-bestfit', '-painters', '-dpdf')

find(arrayfun(@(x) strcmp(x.filename,'b4_jUP_p18psi_t004_dB_xyzpts.csv'),trialInfoStruct))

find(arrayfun(@(x) strcmp(x.filename,'b5_jUP_p18psi_t003_dB_xyzpts.csv'),trialInfoStruct))
find(arrayfun(@(x) strcmp(x.filename,'b5_jUP_p18psi_t003_dF_xyzpts.csv'),trialInfoStruct))



arrayfun(@(x) x.bodyAngleAtBBCrossMidline<32, trialInfoStruct)

bodyAAData  = arrayfun(@(x) x.bodyAngularAccelerationInBodyAtBBCrossMidline, trialInfoStruct);
bodyMBAData = arrayfun(@(x) x.minBodyAngle, trialInfoStruct);
birdIDTrs = arrayfun(@(x) x.birdID, trialInfoStruct, 'UniformOutput', false);
birdNames = {'b1','b2','b4','b5'};
figure()
for i = 1:4
    subplot(4,2,2*i-1)
    histogram(bodyAAData(cellfun(@(x) strcmp(x,birdNames{i}), birdIDTrs)),'NumBins',10)
    axis([-350 250 0 15])
    subplot(4,2,2*i)
    histogram(bodyMBAData(cellfun(@(x) strcmp(x,birdNames{i}), birdIDTrs)),'NumBins',10)
    axis([-40 40 0 15])
end

figure()
for i = 1:length(trialInfoStruct)
    tempWEBZ = trialInfoStruct(i).trialData.tailElevationInBirdZ;
    tempTailExt = trialInfoStruct(i).trialData.tailExtensionAngle;
    tempWEB = trialInfoStruct(i).trialData.tailElevationInBody;
    frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
    lrf = trialInfoStruct(i).lastRealFrame;
    tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidline,trialInfoStruct(i).tTCLeaveGust,lrf]));
    tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGust,lrf]));
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        plotcol = [1 0 0];
        j = 1;
    else
        plotcol = [0 0 1];
        j = 2;
    end
    subplot(2,2,j)
    plot(tempWEBZ(:,2), tempTailExt(:,2), 'Color', 'black');
    plot(tempWEBZ(frf:tcmf,2), tempTailExt(frf:tcmf,2), 'Color', [1 0 0]);
    plot(tempWEBZ(tcmf:tlgf,2), tempTailExt(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot(tempWEBZ(tlgf:lrf,2), tempTailExt(tlgf:lrf,2), 'Color', [0 0 1]);
    hold on
    axis([-100 100 0 90])
    subplot(2,2,2+j)
    plot(tempWEB(:,2), tempTailExt(:,2), 'Color', 'black');
    plot(tempWEB(frf:tcmf,2), tempTailExt(frf:tcmf,2), 'Color', [1 0 0]);
    plot(tempWEB(tcmf:tlgf,2), tempTailExt(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot(tempWEB(tlgf:lrf,2), tempTailExt(tlgf:lrf,2), 'Color', [0 0 1]);
    hold on
    axis([-100 100 0 90])
end

% Does cross tunnel position have any relationship with body angular
% velocity profile?
xLabs = {'Vertical Entry Position (cm)', 'Lateral Entry Position (cm)'};
yLabs = {'Min body elevation angle (deg)', 'Max body angular velocity (deg/s)', 'Min angular velocity (i.e. downward) (deg/s)'};
for i = 1:length(trialInfoStruct)
    zc = trialInfoStruct(i).zBBAtBBCrossMidline;
    yc = trialInfoStruct(i).yBBAtBBCrossMidline;
    minElev = nanmin(trialInfoStruct(i).trialData.birdElevation(:,2));
    maxAngVel = nanmax(trialInfoStruct(i).trialData.bodyAngularVelocityinBody(:,3));
    minAngVel = nanmin(trialInfoStruct(i).trialData.bodyAngularVelocityinBody(:,3));
    
    xs = [zc yc];
    ys = [minElev, maxAngVel, minAngVel];
    
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        plotcol = [1 0 0];
    else
        plotcol = [0 0 1];
    end
    
    for j = 1:length(xs)
        for k = 1:length(ys)
            subplot(3,2,(2*k-2)+j)
            plot(xs(j),ys(k),'o','MarkerSize', 10, 'MarkerFaceColor', plotcol, 'MarkerEdgeColor', 'None');
            hold on
            if j == 2
                axis([-15.24/2 15.24/2 -inf inf])
            end
            xlabel(xLabs{j})
            ylabel(yLabs{k})
        end
    end
    
end


%% EXPLORE WING KINEMATICS

% Looking at wing phase at gust entry (UPDATE ONCE HAVE TECHNIQUE!)
figure()
wingPhaseData = cell2mat(arrayfun(@(x) [strcmp(x.gustTreatment,'18psi'), x.bodyAngularAccelerationInBodyAtBBCrossMidline < -98,  x.wingPhaseAtBBCrossMidline],trialInfoStruct, 'UniformOutput', false));
subplot(1,3,1)
polarhistogram(wingPhaseData(wingPhaseData(:,1)==1 & wingPhaseData(:,2)==1,3),8,'FaceColor','red','FaceAlpha',.3);
subplot(1,3,2)
polarhistogram(wingPhaseData(wingPhaseData(:,1)==1 & wingPhaseData(:,2)==0,3),8,'FaceColor','blue','FaceAlpha',.3);
subplot(1,3,3)
polarhistogram(wingPhaseData(wingPhaseData(:,1)==0,3),8,'FaceColor','black','FaceAlpha',.3);


find(arrayfun(@(x) min(x.trialData.strokeSummaryStruct.amplitude.data(:,2)) < 0,trialInfoStruct))

numFlapsAndMeanFreqInBbCmTcLg = NaN(length(trialInfoStruct),2);
meanFreqBeforeBbCmAndAfTcLg = NaN(length(trialInfoStruct),2);
for i = 1:length(trialInfoStruct)
frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
lrf = trialInfoStruct(i).lastRealFrame;
tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
tempOD = trialInfoStruct(i).trialData.strokeSummaryStruct.frequency.data;
odPos = NaN(size(tempOD,1),3);
for j = 1:size(tempOD,1)
    odPos(j,[1 3]) = tempOD(j,:);
    tpIdx = find(abs(tempPos(:,1)-tempOD(j,1)*500) <= 0.7,1);
    odPos(j,2) = tempPos(tpIdx,2);
end
temp0 = odPos(tempOD(:,1)*500 < frf,[2,3]);
temp1 = odPos(tempOD(:,1)*500 >= frf-1 & tempOD(:,1)*500 < tcmf,[2,3]);
temp2 = odPos(tempOD(:,1)*500 >= tcmf-1 & tempOD(:,1)*500 < tlgf,[2,3]);
temp3 = odPos(tempOD(:,1)*500 >= tlgf-1 & tempOD(:,1)*500 <= lrf,[2,3]);

numFlapsAndMeanFreqInBbCmTcLg(i,:) = [size([temp1;temp2],1)/2, mean([temp1(:,2);temp2(:,2)])];
meanFreqBeforeBbCmAndAfTcLg(i,:) = [mean(temp0(:,2)), mean(temp3(:,2))];

end

nfmg = numFlapsAndMeanFreqInBbCmTcLg(arrayfun(@(x) strcmp(x.gustTreatment,'18psi'), trialInfoStruct),:);
mean(nfmg)
min(nfmg)
max(nfmg)
std(nfmg)/sqrt(size(nfmg,1))
nanmean(arrayfun(@(x) x.tTCLeaveGust, trialInfoStruct))*mean(nfmg(:,2))

nfmg = meanFreqBeforeBbCmAndAfTcLg(arrayfun(@(x) strcmp(x.gustTreatment,'18psi'), trialInfoStruct),:);
[min(nfmg); nanmean(nfmg); max(nfmg)]
nanstd(nfmg)/sqrt(size(nfmg,1)-sum(sum(isnan(nfmg),2)>0))

% Looking at wing sweep offset
figure(1)
featNamesToPlot = {'offset', 'frequency', 'amplitude', 'maxPosAng', 'minPosAng'};
featRanges = {[-90,90], [0,60], [-20,210], [-20,110], [-110,20]};
for featNum = 1:length(featNamesToPlot)
    for i = 1:length(trialInfoStruct)
        frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
        lrf = trialInfoStruct(i).lastRealFrame;
        tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
        tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
        tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
        tempOD = trialInfoStruct(i).trialData.strokeSummaryStruct.(featNamesToPlot{featNum}).data;

        odPos = NaN(size(tempOD,1),3);
        for j = 1:size(tempOD,1)
            odPos(j,[1 3]) = tempOD(j,:);
            tpIdx = find(abs(tempPos(:,1)-tempOD(j,1)*500) <= 0.7,1);
            odPos(j,2) = tempPos(tpIdx,2);
        end

        temp0 = odPos(tempOD(:,1)*500 < frf,[2,3]);
        temp1 = odPos(tempOD(:,1)*500 >= frf-1 & tempOD(:,1)*500 < tcmf,[2,3]);
        temp2 = odPos(tempOD(:,1)*500 >= tcmf-1 & tempOD(:,1)*500 < tlgf,[2,3]);
        temp3 = odPos(tempOD(:,1)*500 >= tlgf-1 & tempOD(:,1)*500 <= lrf,[2,3]);

        if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
            %if trialInfoStruct(i).bodyAngleAtBBCrossMidline< 32
            if trialInfoStruct(i).bodyAngularAccelerationInBodyAtBBCrossMidline < -98
                j = 1;
            else
                j = 2;
            end
        else
            j = 3;
        end
        subplot(length(featNamesToPlot),3,3*(featNum-1)+j)
        plot(temp0(:,1),temp0(:,2), 'Color', [0 0 0 0.9]);
        hold on
        plot(temp1(:,1),temp1(:,2), 'Color', [1 0 0 0.9]);
        plot(temp2(:,1), temp2(:,2), 'Color', [0 1 0 0.9]);
        plot(temp3(:,1), temp3(:,2), 'Color', [0 0 1 0.9]);
        axis([-20 20 featRanges{featNum}])
        title(featNamesToPlot{featNum})
    end
end

% A plot of all wing positions that doesn't tell us much
figure()
for i = 1:length(trialInfoStruct)
    tempWPZ = trialInfoStruct(i).trialData.tFromWBToWTrawPInBirdZ(:,2:4);
    tempWPZ = tempWPZ./sqrt(sum(tempWPZ.^2,2));
    tempWPB = trialInfoStruct(i).trialData.tFromWBToWTrawPInBody(:,2:4);
    tempWPB = tempWPB./sqrt(sum(tempWPB.^2,2));
    frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
    lrf = trialInfoStruct(i).lastRealFrame;
    tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        plotcol = [1 0 0];
        j = 1;
    else
        plotcol = [0 0 1];
        j = 2;
    end
    subplot(2,2,j)
    plot3(tempWPZ(1:frf,1),tempWPZ(1:frf,2),tempWPZ(1:frf,3),'.', 'Color', [0 0 0 0.25]);
    hold on
    plot3(tempWPZ(frf:tcmf,1),tempWPZ(frf:tcmf,2),tempWPZ(frf:tcmf,3),'.', 'Color', [1 0 0 0.25]);
    plot3(tempWPZ(tcmf:tlgf,1), tempWPZ(tcmf:tlgf,2), tempWPZ(tcmf:tlgf,3),'.', 'Color', [0 1 0 0.25]);
    plot3(tempWPZ(tlgf:lrf,1), tempWPZ(tlgf:lrf,2), tempWPZ(tlgf:lrf,3),'.', 'Color', [0 0 1 0.25]);
    axis([-1,1, -1,1, -1,1].*1.5)
    pbaspect([1 1 1]);
    daspect([1 1 1]);
    subplot(2,2,2+j)
    plot3(tempWPB(1:frf,1),tempWPB(1:frf,2),tempWPB(1:frf,3),'.', 'Color', [0 0 0 0.25]);
    hold on
    plot3(tempWPB(frf:tcmf,1),tempWPB(frf:tcmf,2),tempWPB(frf:tcmf,3),'.', 'Color', [1 0 0 0.25]);
    plot3(tempWPB(tcmf:tlgf,1), tempWPB(tcmf:tlgf,2), tempWPB(tcmf:tlgf,3),'.', 'Color', [0 1 0 0.25]);
    plot3(tempWPB(tlgf:lrf,1), tempWPB(tlgf:lrf,2), tempWPB(tlgf:lrf,3),'.', 'Color', [0 0 1 0.25]);
    axis([-1,1, -1,1, -1,1].*1.5)
    pbaspect([1 1 1]);
    daspect([1 1 1]);
end

% Various wing angles
figure()
plotNames = {'Wing sweep angle in BirdZ (deg)',...
    'Wing sweep angle in Body (deg)',...
    'Wing elevation angle in BirdZ (deg)',...
    'Wing elevation angle in Body (deg)'};
for i = 1:length(trialInfoStruct)
    tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
    tempSWBZ = trialInfoStruct(i).trialData.rawWingSweepAngle;
    tempSWB = trialInfoStruct(i).trialData.rawWingSweepAngleInBody;
    tempWEBZ = trialInfoStruct(i).trialData.rawWingElevationAngle;
    tempWEB = trialInfoStruct(i).trialData.rawWingElevationAngleInBody;
    frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
    lrf = trialInfoStruct(i).lastRealFrame;
    tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        plotcol = [1 0 0];
        j = 1;
    else
        plotcol = [0 0 1];
        j = 2;
    end
    subplot(4,2,j)
    hold on
    plot(tempPos(:,2), tempSWBZ(:,2), 'Color', 'black');
    plot(tempPos(frf:tcmf,2), tempSWBZ(frf:tcmf,2), 'Color', [1 0 0]);
    plot(tempPos(tcmf:tlgf,2), tempSWBZ(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot(tempPos(tlgf:lrf,2), tempSWBZ(tlgf:lrf,2), 'Color', [0 0 1]);
    axis([-20 20 -120 120])
    subplot(4,2,2+j)
    hold on
    plot(tempPos(:,2), tempSWB(:,2), 'Color', 'black');
    plot(tempPos(frf:tcmf,2), tempSWB(frf:tcmf,2), 'Color', [1 0 0]);
    plot(tempPos(tcmf:tlgf,2), tempSWB(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot(tempPos(tlgf:lrf,2), tempSWB(tlgf:lrf,2), 'Color', [0 0 1]);
    axis([-20 20 -120 120])
    subplot(4,2,4+j)
    hold on
    plot(tempPos(:,2), tempWEBZ(:,2), 'Color', 'black');
    plot(tempPos(frf:tcmf,2), tempWEBZ(frf:tcmf,2), 'Color', [1 0 0]);
    plot(tempPos(tcmf:tlgf,2), tempWEBZ(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot(tempPos(tlgf:lrf,2), tempWEBZ(tlgf:lrf,2), 'Color', [0 0 1]);
    axis([-20 20 -120 120])
    subplot(4,2,6+j)
    hold on
    plot(tempPos(1:frf,2), tempWEB(1:frf,2), 'Color', 'black');
    plot(tempPos(frf:tcmf,2), tempWEB(frf:tcmf,2), 'Color', [1 0 0 0.5]);
    plot(tempPos(tcmf:tlgf,2), tempWEB(tcmf:tlgf,2), 'Color', [0 1 0 0.5]);
    plot(tempPos(tlgf:lrf,2), tempWEB(tlgf:lrf,2), 'Color', [0 0 1 0.5]);
    %axis([-20 20 -120 120])
    axis([-20 20 -inf inf])
end
for i = 1:length(plotNames)
    subplot(4,2,2*i-1)
    title(plotNames(i))
    subplot(4,2,2*i)
    title(plotNames(i))
end


% A SEPARATED PLOT OF WING ANGLES
fig = figure(3);
featNamesToPlot = {'rawWingSweepAngle', 'rawWingSweepAngleInBody','rawWingElevationAngle',...
    'rawWingElevationAngleInBody','tailExtensionAngle'};
for featNum = 1:length(featNamesToPlot)
    if contains(featNamesToPlot{featNum},'Angular')
        dataIdx = 3;
    else
        dataIdx = 2;
    end
    ymin = nanmin(arrayfun(@(x) nanmin(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));
    ymax = nanmax(arrayfun(@(x) nanmax(x.trialData.(featNamesToPlot{featNum})(:,dataIdx)),trialInfoStruct));

    for i = 1:length(trialInfoStruct)
        frf = round(trialInfoStruct(i).firstRealFrame);
        lrf = trialInfoStruct(i).lastRealFrame;

        bent = round(trialInfoStruct(i).tBBEnterGustFrameNum);
        bbcm = round(trialInfoStruct(i).zeroCrossingFrameNums);
        tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
        tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));

        tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
        tempData = trialInfoStruct(i).trialData.(featNamesToPlot{featNum});

        if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
            %if trialInfoStruct(i).bodyAngleAtBBCrossMidline< 32
            if trialInfoStruct(i).bodyAngularAccelerationInBodyAtBBCrossMidline < -98*180/pi
                j = 1;
            else
                j = 2;
            end
        else
            j = 3;
        end
        if j < 3
            subplot(length(featNamesToPlot),3,3*(featNum-1)+1)
            plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            hold on
            axis([-20 20 ymin ymax])
            subplot(length(featNamesToPlot),3,3*(featNum-1)+2)
            plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0.5 0.5 0.5 0.1]);
            hold on
            axis([-20 20 ymin ymax])
        end
        subplot(length(featNamesToPlot),3,3*(featNum-1)+j)
        plot(tempPos(frf:bent,2), tempData(frf:bent,dataIdx), 'Color', 'black');
        plot(tempPos(bent:bbcm,2), tempData(bent:bbcm,dataIdx), 'Color', [255,128,0]/255);
        plot(tempPos(bbcm:tcmf,2), tempData(bbcm:tcmf,dataIdx), 'Color', [1 0 0]);
        plot(tempPos(tcmf:tlgf,2), tempData(tcmf:tlgf,dataIdx), 'Color', [0 1 0]);
        plot(tempPos(tlgf:lrf,2), tempData(tlgf:lrf,dataIdx), 'Color', [0 0 1]);
        hold on
        axis([-20 20 ymin ymax])
    end
end
for i = 1:length(featNamesToPlot)
    subplot(5,3,3*(i-1)+1)
    title(["Gust: pitch accel < -98 " featNamesToPlot(i)])
    subplot(5,3,3*(i-1)+2)
    title(["Gust: pitch accel > -98 " featNamesToPlot(i)])
    subplot(5,3,3*(i-1)+3)
    title(["Control: " featNamesToPlot(i)])
end

orient(fig,'landscape')
fig.PaperPositionMode = 'manual';
print(fig, 'output.pdf', '-bestfit', '-painters', '-dpdf')


% A 3D plot of wing elevation and sweep angle vs position
figure()
plotNames = {'Wing sweep and elevation angle in BirdZ (deg)'};
for i = 1:length(trialInfoStruct)
    tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
    tempSWBZ = trialInfoStruct(i).trialData.rawWingSweepAngle; % rawWingSweepAngleInBody;
    tempWEBZ = trialInfoStruct(i).trialData.rawWingElevationAngle; %rawWingElevationAngleInBody;
    frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
    lrf = trialInfoStruct(i).lastRealFrame;
    tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        plotcol = [1 0 0];
        j = 1;
    else
        plotcol = [0 0 1];
        j = 2;
    end
    subplot(1,2,j)
    plot3(tempPos(1:frf,2), tempSWBZ(1:frf,2), tempWEBZ(1:frf,2), 'Color', 'black');
    plot3(tempPos(frf:tcmf,2), tempSWBZ(frf:tcmf,2), tempWEBZ(frf:tcmf,2), 'Color', [1 0 0]);
    plot3(tempPos(tcmf:tlgf,2), tempSWBZ(tcmf:tlgf,2), tempWEBZ(tcmf:tlgf,2), 'Color', [0 1 0]);
    plot3(tempPos(tlgf:lrf,2), tempSWBZ(tlgf:lrf,2), tempWEBZ(tlgf:lrf,2), 'Color', [0 0 1]);
    hold on
    axis([-20 20 -120 120 -120 120])
    xlabel('Tunnel position')
    ylabel('Wing sweep')
    zlabel('Wing elevation')
    pbaspect([0.1 1 1]);
    daspect([0.1 1 1]);
end
for i = 1:length(plotNames)
    subplot(1,2,2*i-1)
    title(plotNames(i))
    subplot(1,2,2*i)
    title(plotNames(i))
end

figure()
wingPhasevsAngAccel = cell2mat(arrayfun(@(x) [x.wingPhaseAtBBCrossMidline, x.bodyAngularAccelerationInBodyAtBBCrossMidline], trialInfoStruct, 'UniformOutput',false));
polarplot(wingPhasevsAngAccel(:,1)*pi/180,wingPhasevsAngAccel(:,2),'.','MarkerSize',20)

% WING ANGLES COLORED BY PC1
figure()
plotNames = {'Wing sweep angle in BirdZ (deg)',...
    'Wing sweep angle in Body (deg)',...
    'Wing elevation angle in BirdZ (deg)',...
    'Wing elevation angle in Body (deg)'};
%sortValue = pc1sForTrialInfoStruct(:,1);
sortValue = arrayfun(@(x) x.bodyAngularAccelerationInBodyAtBBCrossMidline,trialInfoStruct);
[~, orderIdx] = sort(sortValue);
[~, orderIdx] = sort(orderIdx);
sortNumber = orderIdx/max(orderIdx);
sortNumber(sortNumber < 0.1) = 0;
sortNumber(sortNumber > 0.7) = 1;
sortNumber(sortNumber >= 0.1 & sortNumber <= 0.7) = 0.1;
for i = 1:length(trialInfoStruct)
    tempPos = trialInfoStruct(i).trialData.tFromANCToBBsmoothedPVAInLab;
    tempSWBZ = trialInfoStruct(i).trialData.rawWingSweepAngle;
    tempSWB = trialInfoStruct(i).trialData.rawWingSweepAngleInBody;
    tempWEBZ = trialInfoStruct(i).trialData.rawWingElevationAngle;
    tempWEB = trialInfoStruct(i).trialData.rawWingElevationAngleInBody;
    frf = round(trialInfoStruct(i).zeroCrossingFrameNums);
    lrf = trialInfoStruct(i).lastRealFrame;
    tcmf = round(nanmin([trialInfoStruct(i).tTCCrossMidlineFrameNum,trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    tlgf = round(nanmin([trialInfoStruct(i).tTCLeaveGustFrameNum,lrf]));
    if strcmp(trialInfoStruct(i).gustTreatment,'18psi')
        j = 1;
    else
        j = 2;
    end
    if sortNumber(i) == 0
        segPlotColors = [2 136 217; 7 185 252; 0 149 67; 0 171 56]/255;
        segPlotColors = [segPlotColors ones(4,1)];
    elseif sortNumber(i) == 1
        segPlotColors = [253 71 3; 255 130 42; 130 0 172; 182 16 191]/255;
        segPlotColors = [segPlotColors ones(4,1)];
    else
        segPlotColors = [0.1 0.1 0.1; 0.4 0.4 0.4; 0.2 0.2 0.2; 0.1 0.1 0.1];
        segPlotColors = [segPlotColors 0.1*ones(4,1)];
    end
    subplot(4,2,j)
    plot(tempPos(:,2), tempSWBZ(:,2), 'Color', segPlotColors(1,:));
    plot(tempPos(frf:tcmf,2), tempSWBZ(frf:tcmf,2), 'Color', segPlotColors(2,:));
    plot(tempPos(tcmf:tlgf,2), tempSWBZ(tcmf:tlgf,2), 'Color', segPlotColors(3,:));
    plot(tempPos(tlgf:lrf,2), tempSWBZ(tlgf:lrf,2), 'Color', segPlotColors(4,:));
    hold on
    axis([-20 20 -120 120])
    subplot(4,2,2+j)
    plot(tempPos(:,2), tempSWB(:,2), 'Color', segPlotColors(1,:));
    plot(tempPos(frf:tcmf,2), tempSWB(frf:tcmf,2), 'Color', segPlotColors(2,:));
    plot(tempPos(tcmf:tlgf,2), tempSWB(tcmf:tlgf,2), 'Color', segPlotColors(3,:));
    plot(tempPos(tlgf:lrf,2), tempSWB(tlgf:lrf,2), 'Color', segPlotColors(4,:));
    hold on
    axis([-20 20 -120 120])
    subplot(4,2,4+j)
    plot(tempPos(:,2), tempWEBZ(:,2), 'Color', segPlotColors(1,:));
    plot(tempPos(frf:tcmf,2), tempWEBZ(frf:tcmf,2), 'Color', segPlotColors(2,:));
    plot(tempPos(tcmf:tlgf,2), tempWEBZ(tcmf:tlgf,2), 'Color', segPlotColors(3,:));
    plot(tempPos(tlgf:lrf,2), tempWEBZ(tlgf:lrf,2), 'Color', segPlotColors(4,:));
    hold on
    axis([-20 20 -120 120])
    subplot(4,2,6+j)
    plot(tempPos(1:frf,2), tempWEB(1:frf,2), 'Color', segPlotColors(1,:));
    plot(tempPos(frf:tcmf,2), tempWEB(frf:tcmf,2), 'Color', segPlotColors(2,:));
    plot(tempPos(tcmf:tlgf,2), tempWEB(tcmf:tlgf,2), 'Color',segPlotColors(3,:));
    plot(tempPos(tlgf:lrf,2), tempWEB(tlgf:lrf,2), 'Color', segPlotColors(4,:));
    hold on
    %axis([-20 20 -120 120])
    axis([-20 20 -inf inf])
end
for i = 1:length(plotNames)
    subplot(4,2,2*i-1)
    title(plotNames(i))
    subplot(4,2,2*i)
    title(plotNames(i))
end





%% Calculating wingspan estimates

birdIDs = unique(arrayfun(@(x) x.birdID, trialInfoStruct, 'UniformOutput', false));
infoByBird = cell(length(birdIDs),1);
for i = 1:length(birdIDs)
    idxs = arrayfun(@(x) strcmp(x.birdID, birdIDs{i}), trialInfoStruct);
    infoByBird{i} = trialInfoStruct(idxs);
end

% 2 [10 15 17]
infoByBird{2}(17) = [];
infoByBird{2}(15) = [];
infoByBird{2}(10) = [];

birdInfo = struct('birdID', birdIDs);
for i = 1:length(birdIDs)
    wingLengths = cell2mat(arrayfun(@(x) x.wingLengthsCM, infoByBird{i}, 'UniformOutput',false));
    wingSpans = cell2mat(arrayfun(@(x) x.peakWingSpansCM, infoByBird{i}, 'UniformOutput',false));
    tsstSpans = cell2mat(arrayfun(@(x) x.tipShShTipSpansCM, infoByBird{i}, 'UniformOutput',false));
    
    maxLengthsByTrial = cell2mat(arrayfun(@(x) max(x.wingLengthsCM), infoByBird{i}, 'UniformOutput',false));
    maxSpansByTrial = cell2mat(arrayfun(@(x) max(x.peakWingSpansCM), infoByBird{i}, 'UniformOutput',false));
    maxtsstsByTrial = cell2mat(arrayfun(@(x) max(x.tipShShTipSpansCM), infoByBird{i}, 'UniformOutput',false));

    birdInfo(i).medianWingLength = nanmedian(wingLengths);
    birdInfo(i).meanWingLength = nanmean(wingLengths);
    birdInfo(i).medianMaxWingLength = nanmedian(maxLengthsByTrial);
    birdInfo(i).meanMaxWingLength = nanmean(maxLengthsByTrial);
    birdInfo(i).medianWingSpan = nanmedian(wingSpans);
    birdInfo(i).meanWingSpan = nanmean(wingSpans);
    birdInfo(i).medianMaxWingSpan = nanmedian(maxSpansByTrial);
    birdInfo(i).meanMaxWingSpan = nanmean(maxSpansByTrial);
    
    birdInfo(i).mediantsstSpan = nanmedian(tsstSpans);
    birdInfo(i).meantsstSpan = nanmean(tsstSpans);
    birdInfo(i).medianMaxtsstSpan = nanmedian(maxtsstsByTrial);
    birdInfo(i).meanMaxtsstSpan = nanmean(maxtsstsByTrial);
    birdInfo(i).SDtsstSpan = nanstd(tsstSpans);
end

% Standard error using mean of each trial as unit of measurement
for i = 1:length(birdIDs)
    meanWLs = arrayfun(@(x) nanmean(x.tipShShTipSpansCM),infoByBird{i});
    stdMean = nanstd(meanWLs);
    seMean = stdMean/sqrt(length(meanWLs));
    
    birdInfo(i).SEForMeanOfTrialMeans = seMean;
end

plotColors = {'red','green','blue','black'};
figure()
for i = 1:length(birdIDs)
    subplot(2,2,i)
    hold on
    for trialNum = 1:length(infoByBird{i})
        ys = infoByBird{i}(trialNum).tipShShTipSpansCM;
        xs = ones(length(ys),1).*trialNum;
        plot(xs, ys, [plotColors{i}, 'o']);
    end
end


b1Ws = cell2mat(arrayfun(@(x) x.tipShShTipSpansCM, infoByBird{1}, 'UniformOutput',false));
b2Ws = cell2mat(arrayfun(@(x) x.tipShShTipSpansCM, infoByBird{2}, 'UniformOutput',false));
b3Ws = cell2mat(arrayfun(@(x) x.tipShShTipSpansCM, infoByBird{3}, 'UniformOutput',false));
b4Ws = cell2mat(arrayfun(@(x) x.tipShShTipSpansCM, infoByBird{4}, 'UniformOutput',false));

max([size(b1Ws,1), size(b2Ws,1), size(b3Ws,1), size(b4Ws,1)])

allWs = NaN(3377,4);
allWs(1:size(b1Ws,1),1) = b1Ws;
allWs(1:size(b2Ws,1),2) = b2Ws;
allWs(1:size(b3Ws,1),3) = b3Ws;
allWs(1:size(b4Ws,1),4) = b4Ws;

boxplot(allWs)

[birdInfo.meantsstSpan]
[birdInfo.SEForMeanOfTrialMeans]
    
struct2table(birdInfo)


%     birdID    medianWingLength    meanWingLength    medianMaxWingLength    meanMaxWingLength    medianWingSpan    meanWingSpan    medianMaxWingSpan    meanMaxWingSpan    mediantsstSpan    meantsstSpan    medianMaxtsstSpan    meanMaxtsstSpan    SDtsstSpan    SEForMeanOfTrialMeans
%     ______    ________________    ______________    ___________________    _________________    ______________    ____________    _________________    _______________    ______________    ____________    _________________    _______________    __________    _____________________
% 
%     'b1'      4.9748              4.9768            5.6966                 5.7149                11.47             11.16          12.271               12.392             11.671            11.653          13.185               13.131             0.68836        0.10197             
%     'b2'      4.8817              4.8764            5.6781                 5.7376               11.037            10.904           11.81               11.806             11.233            11.191          12.787               12.919             0.85897       0.050075             
%     'b4'      4.9461              4.9363            5.4807                 5.4915               11.115            11.048           11.51               11.483             11.098            11.075          12.272               12.252             0.58505       0.026611             
%     'b5'      4.9238              4.9175            5.6926                 5.7092               11.667            11.403          12.292               12.264             11.802            11.786          13.432               13.441             0.85493       0.041387
