function gustPositionStruct = prepCalibrations(workingDirectory, createFigures)
%prepCalibrations - loads annotated features on gust generator from csv
%file, calculates rotations between the calibration frame and gust
%generator/lab frame and the vector from the origin of the calibration
%frame to the center of the gust generator
%
% Inputs:
%    working directory - directory containing a folder 'DLT Coefficient',
%    which contains a folder 'AirKnifePoints', which contains a xyzpts.csv
%    file generated using DLTdv5 (Hedrick 2008) for each calibration.
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    gustPositionStruct - a struct containing fields:
%       birdName: the bird name (which for our experiment uniquely identifies the calibration)
%       pointPositions: the 3D positions of the features on the gust generator
%       tFromOToANCinCD: the vector from the origin of the calibration frame to the center of the gust generator
%       RFromCDToAN: the 3D rotation matrix from the calibration frame to the gust generator/lab frame
%           (x is along the axis of the tunnel, y is transverse to the
%           tunnel such that z is vertical and points opposite the
%           direction of gravity)
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

gustPositionFilenames = dir(fullfile(workingDirectory,...
    'DLT Coefficient','AirKnifePoints','b*xyzpts.csv'));

correspBirdNames = arrayfun(@(x) x.name(1:2), gustPositionFilenames, 'UniformOutput',false);
pointPositions(1:length(correspBirdNames)) = {ones(10,3)}; % hard coded based on annotated points in file
gustPositionStruct = struct('birdName', correspBirdNames', 'pointPositions', pointPositions);

for i = 1:length(gustPositionFilenames)
    [~, ~, rawGust] = xlsread(fullfile(gustPositionFilenames(i).folder,...
        gustPositionFilenames(i).name));
    gustPositionStruct(i).pointPositions = reshape(cell2mat(rawGust(2,:)),3,[])';
end

if createFigures
    figure()
    for i = 1:length(gustPositionStruct)
        % The gust generator was aligned to within 1-3 mm of directly
        % transverse to the flight arena and calibration shape.
        % Mean vector from one side of the gust opening to the other
        disp(mean(gustPositionStruct(i).pointPositions([2 4 6 8],:)-...
            gustPositionStruct(i).pointPositions([1 3 5 7],:)))

        subplot(2,2,i)
        points = gustPositionStruct(i).pointPositions;
        plot3(points(:,1), points(:,2), points(:,3), 'ro')
        xlabel('x')
        ylabel('y')
        zlabel('z')
        pbaspect([1 1 1]);
        daspect([1 1 1]);
    end
end

% tFromOToANCinCDs = cell(length(gustPositionStruct),1);
% RFromCDToANs = cell(length(gustPositionStruct),1);

for i = 1:length(gustPositionStruct)
    gustHorizVector = mean(gustPositionStruct(i).pointPositions([2 4 6 8],:)-...
        gustPositionStruct(i).pointPositions([1 3 5 7],:));
    gustHorizVector(3) = 0; % we only want to rotate about the z axis (cameras
                            % were aligned to gravity using a bubble level).
    gustPositionStruct(i).tFromOToANCinCD = mean(gustPositionStruct(i).pointPositions(1:2,:));
    
    % The vector along the gust points in the -y direction transverse to
    % the cage.
    labHorizVector = [0 -1 0];
    rot = vrrotvec(labHorizVector, gustHorizVector);
    gustPositionStruct(i).RFromCDToAN = vrrotvec2mat(rot);
end

end