function infoStructEntry = rotateDataToBirdFrame(infoStructEntry, frameRate, createFigures)
%rotateDataToBirdFrame - rotates data from the lab frame to the body
%frame. The bird body frame is defined by body_x = vector from rump to bill
%base, body_y perpendicular to body_x with dot(body_y, lab_z) = 0, and
%body_z = cross(body_x, body_y).  In practice, the body frame is simply
%rotated about birdZ_y (pitched up/down) until body_x is aligned with the
%vector from rump to bill base. This function and rotateDataToBirdZFrame
%correspond to finding the yaw and pitch Euler angles. Note we cannot
%estimate the roll of the bird based on the tracked data and therefore make
%the assumption that it is unchanged/nominal - this may have minor effects
%on the body-relative estimates of wing sweep and elevation in the fixed
%body frame. This function als calls calculateBodyAngularVelocity.m to
% calculate the body angular velocity and acceleration in the lab and 
% fixed body frames.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing rotated kinematics data
%    into the body frame.  Also caluclates the body angular velocity and
%    acceleration in the lab and fixed body frames
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

% Body frame x is aligned with with bird's rump to bill base axis and
% the origin is the bird's bill base. 

tds = infoStructEntry.trialData;

tdsFields = fields(tds);
featureNamesToRotate = tdsFields(cellfun(@(x) contains(x,'PInBirdZ'),tdsFields));

rotatedFeatureNames = cellfun(@(x)...
    strrep(x,'PInBirdZ','PInBody'), featureNamesToRotate,'UniformOutput',false);

for featureNum = 1:length(featureNamesToRotate)
        tds.(rotatedFeatureNames{featureNum}) =...
            tds.(featureNamesToRotate{featureNum});
end

tFromBBtoBBinBirdZ = tds.tFromBBToBBsmoothedPInBirdZ(:,2:4);
tFromBBtoRUinBirdZ = tds.tFromBBToRUsmoothedPInBirdZ(:,2:4);
%     
%     figure
%     plot3(tFromBBtoBBinBirdZ(:,1),tFromBBtoBBinBirdZ(:,2),tFromBBtoBBinBirdZ(:,3),'ro')
%     hold on
%     plot3(tFromBBtoRUinBirdZ(:,1),tFromBBtoRUinBirdZ(:,2),tFromBBtoRUinBirdZ(:,3),'bo')
%     xlabel('x')
%     ylabel('y')
%     zlabel('z')
%     pbaspect([1 1 1]);
%     daspect([1 1 1]);


% Transform to a new coordinate system aligned with body_x (birdZ_y = body_y), but
% centered on the bird's bill base (want positions as tFromBBToXYZinBody).
allRFromBirdZToBody = ones(3,3,size(tFromBBtoBBinBirdZ,1));
allRFromLabToBody = ones(3,3,size(tFromBBtoBBinBirdZ,1));
alltFromBBToBBinBody = ones(size(tFromBBtoBBinBirdZ,1),3);

for frameNum=1:size(tFromBBtoBBinBirdZ,1)
    % RumpToBillBase = RumpToOrigin + OriginToBillBase = OriginToBillBase-OriginToRump;
    tFromRUToBBinBirdZ = tFromBBtoBBinBirdZ(frameNum,:)-tFromBBtoRUinBirdZ(frameNum,:);
    
    if sum(isnan(tFromRUToBBinBirdZ))>0
        for featureNum = 1:length(featureNamesToRotate)
            tds.(rotatedFeatureNames{featureNum})(frameNum,2:4) = [NaN, NaN, NaN];
        end
    else
        if ~isnan(tFromRUToBBinBirdZ(2))
            tFromRUToBBinBirdZ(2) = 0; % only rotate about the y axis
        end
        tFromRUToBBinBody = [1 0 0];
        rot = vrrotvec(tFromRUToBBinBody, tFromRUToBBinBirdZ);
        RFromBirdZToBody = vrrotvec2mat(rot);
        allRFromBirdZToBody(:,:,frameNum) = RFromBirdZToBody;
        allRFromLabToBody(:,:,frameNum) = tds.RFromLabToBirdZ(:,:,frameNum)*RFromBirdZToBody;

        tFromBBToXYZinBirdZ = cellfun(@(x) tds.(x)(frameNum,:),featureNamesToRotate,'UniformOutput',false);
        tFromBBToXYZinBirdZ = cellfun(@(x) x(:,2:4), tFromBBToXYZinBirdZ,'UniformOutput',false);
        tFromBBToXYZinBirdZ = cell2mat(tFromBBToXYZinBirdZ);

        alltFromBBToBBinBody(frameNum,:) = tFromBBtoBBinBirdZ(frameNum,:);

        tFromBBToXYZinBody = tFromBBToXYZinBirdZ*RFromBirdZToBody;

        for featureNum = 1:length(featureNamesToRotate)
            tds.(rotatedFeatureNames{featureNum})(frameNum,2:4) = tFromBBToXYZinBody(featureNum,:);
        end
    end
    
end

tds.RFromBirdZToBody = allRFromBirdZToBody;
tds.RFromLabToBody = allRFromLabToBody;

[bodyAngularVelocityinBody, bodyAngularAccelerationinBody, bodyAngularJerkinBody] = calculateBodyAngularVelocity(tds.RFromLabToBody, frameRate);

% Convert to lab frame too
bodyAngularVelocityinLab = bodyAngularVelocityinBody;
bodyAngularAccelerationinLab = bodyAngularAccelerationinBody;
bodyAngularJerkinLab = bodyAngularJerkinBody;
for i = 1:size(bodyAngularVelocityinLab)
    bodyAngularVelocityinLab(i,:) = bodyAngularVelocityinBody(i,:)*tds.RFromLabToBody(:,:,i)';
    bodyAngularAccelerationinLab(i,:) = bodyAngularAccelerationinBody(i,:)*tds.RFromLabToBody(:,:,i)';
    bodyAngularJerkinLab(i,:) = bodyAngularJerkinBody(i,:)*tds.RFromLabToBody(:,:,i)';
end

tds.bodyAngularVelocityinBody = [tds.frameNum, bodyAngularVelocityinBody];
tds.bodyAngularAccelerationinBody = [tds.frameNum, bodyAngularAccelerationinBody];
tds.bodyAngularJerkinBody = [tds.frameNum, bodyAngularJerkinBody];

tds.bodyAngularVelocityinLab = [tds.frameNum, bodyAngularVelocityinLab];
tds.bodyAngularAccelerationinLab = [tds.frameNum, bodyAngularAccelerationinLab];
tds.bodyAngularJerkinLab = [tds.frameNum, bodyAngularJerkinLab];


if createFigures
    for i = 1:3
        subplot(2,3,i)
        plot(bodyAngularVelocityinBody(:,i))
        hold on
        plot(bodyAngularVelocityinLab(:,i))
        hold off
    end
    for i = 1:3
        subplot(2,3,i+3)
        plot(bodyAngularAccelerationinBody(:,i))
        hold on
        plot(bodyAngularAccelerationinLab(:,i))
        hold off
    end
end

%% Calculate tail elevation angle in the body frame
tFromRUToTCinBody = -tds.tFromBBToRUsmoothedPInBody(:,2:4) + tds.tFromBBToTCsmoothedPInBody(:,2:4);
% reflect the x direction to make cart2sph output the angle we want
[~,elevation,~] = cart2sph(-tFromRUToTCinBody(:,1),tFromRUToTCinBody(:,2),tFromRUToTCinBody(:,3));
tailElevations(:,2) = elevation*180/pi;
tds.tailElevationInBody = tailElevations;

infoStructEntry.trialData = tds;

end







