function infoStructEntry = rotateDataToBirdZFrame(infoStructEntry, createFigures)
%rotateDataToBirdZFrame - rotates data from the lab frame to the birdZ
%frame. The birdZ frame is defined by birdZ_z = lab_z and birdZ_x = vector
%from rump to bill base projected onto lab_x-lab_y plane. Data are also
%translated so that the origin is at the bill base.
%
% Inputs:
%    infoStructEntry - an entry of a struct array containing data for the desired trial
%    createFigures - whether figures should be displayed for inspection
%
% Outputs:
%    infoStructEntry - an updated entry containing rotated kinematics data
%    into the birdZ frame. Also calculates the body's heading and elevation
%    angles.
%
% Author: Marc Badger
% email: badger@berkeley.edu
% May 2018; Last revision: 21-May-2018

tds = infoStructEntry.trialData;

%% Convert To BirdZ
featureNamesToRotate = fields(tds);
rotateFeatureIdxs = cellfun(@(x) (contains(x,'smoothed') | contains(x,'WT') | contains(x,'FF')) & contains(x,'InLab'), featureNamesToRotate);
featureNamesToRotate = featureNamesToRotate(rotateFeatureIdxs);
rotatedFeatureNames = cellfun(@(x)...
    strrep(strrep(strrep(x,'InLab','InBirdZ'),'PVA','P'),'FromANC','FromBB'), featureNamesToRotate,'UniformOutput',false);

billbase = tds.tFromANCToBBsmoothedPVAInLab(:,2:4);
rump = tds.tFromANCToRUsmoothedPVAInLab(:,2:4);
allRFromLabToBirdZ = ones(3,3,size(billbase,1));
alltFromLabToBBInLab = ones(size(billbase,1),3);
birdElevations = repmat(tds.frameNum,1,2); % just frame numbers
birdHeadings = birdElevations; % copy frame numbers to birdBBings too.
birdLengths = birdElevations;
tailElevations = birdElevations;
tailExtensions = birdElevations;

for frameNumber=1:size(billbase,1)
    % RUToBB = RUToLab + LabToBB = LabToBB-LabToRU;
    tFromRUToBBInLab = billbase(frameNumber,:)-rump(frameNumber,:);
    
    %disp(frameNum)
    
    if sum(isnan(tFromRUToBBInLab))>0
        for featureNum = 1:length(featureNamesToRotate)
            tds.(rotatedFeatureNames{featureNum})(frameNumber,:) = [tds.frameNum(frameNumber), NaN, NaN, NaN];
        end
        birdElevations(frameNumber,2) = NaN;
        birdHeadings(frameNumber,2) = NaN;
        birdLengths(frameNumber,2) = NaN;
    else
        % an aside to get bird elevation and BBing
        [heading,elevation,r] = cart2sph(tFromRUToBBInLab(1),tFromRUToBBInLab(2),tFromRUToBBInLab(3));
        birdElevations(frameNumber,2) = elevation*180/pi;
        birdHeadings(frameNumber,2) = heading*180/pi;
        birdLengths(frameNumber,2) = r;
        
        if ~isnan(tFromRUToBBInLab(3))
            tFromRUToBBInLab(3) = 0; % only rotate about the z axis
        end
        tFromRUToBBinBirdZ = [1 0 0];
        rot = vrrotvec(tFromRUToBBinBirdZ, tFromRUToBBInLab);
        RFromLabToBirdZ = vrrotvec2mat(rot);
        allRFromLabToBirdZ(:,:,frameNumber) = RFromLabToBirdZ;

        tFromANCToXYZInLab = cellfun(@(x) tds.(x)(frameNumber,2:4),featureNamesToRotate,'UniformOutput',false);
        tFromANCToXYZInLab = cell2mat(tFromANCToXYZInLab);

        tFromANCToBBInLab = billbase(frameNumber,:);
        alltFromLabToBBInLab(frameNumber,:) = tFromANCToBBInLab;

        % Derivation of tFromBBToXYZinBirdZ:
    %         tFromBBToXYZinBirdZ = tFromBBToLabinBirdZ + tFromLabToXYZinBirdZ
    %             = tFromLabToXYZInLab*RFromLabToBirdZ - tFromLabToBBinBirdZ
    %             = (tFromLabToXYZInLab-tFromLabToBBInLab)*RFromLabToBirdZ
        tFromBBToXYZinBirdZ = (tFromANCToXYZInLab-tFromANCToBBInLab)*RFromLabToBirdZ;

        for featureNum = 1:length(featureNamesToRotate)
            tds.(rotatedFeatureNames{featureNum})(frameNumber,:) = [tds.frameNum(frameNumber), tFromBBToXYZinBirdZ(featureNum,:)];
        end
    end
end

%% Compute wing length and wingspan
tds.tFromWBToWTrawPInBirdZ = -tds.tFromBBToWBsmoothedPInBirdZ + tds.tFromBBToWTrawPInBirdZ;
tds.tFromWBToWTrawPInBirdZ(:,1) = tds.frameNum;
tds.wingLength = sqrt(sum(tds.tFromWBToWTrawPInBirdZ.^2,2));
tds.tipToTip = abs(2*tds.tFromBBToWTrawPInBirdZ(:,2));
tds.tipShShTipSpan = 2*tds.wingLength + abs(2*tds.tFromBBToWBsmoothedPInBirdZ(:,2));

tds.RFromLabToBirdZ = allRFromLabToBirdZ;
tds.birdHeading = birdHeadings;
tds.birdElevation = birdElevations;

[pks, locs] = findpeaks(tds.tipToTip,'MinPeakDistance',10);

if createFigures
    plot(tds.tipToTip)
    hold on
    plot(locs,pks,'ro')

    plot(tds.wingLength)
end

%% Calculate tail angles
tFromRUToTCinBirdZ = -tds.tFromBBToRUsmoothedPInBirdZ(:,2:4) + tds.tFromBBToTCsmoothedPInBirdZ(:,2:4);
tFromRUToTTinBirdZ = -tds.tFromBBToRUsmoothedPInBirdZ(:,2:4) + tds.tFromBBToTTsmoothedPInBirdZ(:,2:4);
% reflect the x direction to make cart2sph output the angle we want
[~,elevation,~] = cart2sph(-tFromRUToTCinBirdZ(:,1),tFromRUToTCinBirdZ(:,2),tFromRUToTCinBirdZ(:,3));
tailElevations(:,2) = elevation*180/pi;
tds.tailElevationInBirdZ = tailElevations;

% Tail extension angle is the angle between the vectors from tail root to tail center and
% from tail root to tail outer tip (i.e. the angle in the plane of the tail
% rather than out from the BirdZ_x or Bird_x axes (prevents extreme sensitivity
% to slight changes in tail tip position when the tail is elevated near 90 deg).
for i = 1:size(tFromRUToTCinBirdZ,1)
    tailExtensions(i,2) = nanvrrotvecAng(tFromRUToTCinBirdZ(i,:), tFromRUToTTinBirdZ(i,:))*180/pi;
end
tds.tailExtensionAngle = tailExtensions;

infoStructEntry.trialData = tds;
infoStructEntry.wingLengthsCM = tds.wingLength(infoStructEntry.firstRealFrame:infoStructEntry.lastRealFrame);
infoStructEntry.peakWingSpansCM = pks;
infoStructEntry.tipShShTipSpansCM = tds.tipShShTipSpan(infoStructEntry.firstRealFrame:infoStructEntry.lastRealFrame);

end